package com.bocloud.cos.booter.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.common.utils.ResultTools;
import com.bocloud.cos.common.enums.FlowConstants;
import com.bocloud.cos.common.model.BasicDomainModel;
import com.bocloud.cos.engine.AssignmentHandler;
import com.bocloud.cos.engine.FlowEngine;
import com.bocloud.cos.engine.model.AuditTaskModel;
import com.bocloud.cos.engine.model.BaseNodeModel;
import com.bocloud.cos.engine.model.DefinitionModel;
import com.bocloud.cos.engine.model.FlowNodeModel;
import com.bocloud.cos.entity.Attachment;
import com.bocloud.cos.entity.FlowInstance;
import com.bocloud.cos.entity.FlowNode;
import com.bocloud.cos.intf.AttachmentService;
import com.bocloud.cos.intf.CloudServiceService;
import com.bocloud.cos.intf.FlowQueryService;
import com.bocloud.cos.model.FlowInstancePatchParam;
import com.bocloud.cos.repository.CloudServiceRepository;
import com.bocloud.cos.repository.OrderItemRepository;
import com.bocloud.cos.repository.ServiceFlowConfigRepository;
import com.bocloud.cos.service.Execution;
import com.google.common.collect.Lists;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * @author hufy
 */
@RestController
@Api(value = "流程实例", tags = "流程实例")
@RequestMapping({"/v1/instances"})
public class FlowInstanceController extends FlowBaseController {

    private final FlowEngine engine;
    private final AssignmentHandler assignmentHandler;
    private final AttachmentService attachmentService;
    private final FlowQueryService flowQueryService;
    private final CloudServiceService cloudServiceService;
    private final CloudServiceRepository cloudServiceRepository;
    private final ServiceFlowConfigRepository serviceFlowConfigRepository;
    private final OrderItemRepository orderItemRepository;


    @Value("${upload.storage.path:/home/cmp}")
    private String path;

    @Autowired
    public FlowInstanceController(FlowEngine engine, AssignmentHandler assignmentHandler, AttachmentService attachmentService,
                                  FlowQueryService flowQueryService, CloudServiceService cloudServiceService,
                                  CloudServiceRepository cloudServiceRepository, ServiceFlowConfigRepository serviceFlowConfigRepository,
                                  OrderItemRepository orderItemRepository) {
        this.engine = engine;
        this.assignmentHandler = assignmentHandler;
        this.attachmentService = attachmentService;
        this.flowQueryService = flowQueryService;
        this.cloudServiceService = cloudServiceService;
        this.cloudServiceRepository = cloudServiceRepository;
        this.serviceFlowConfigRepository = serviceFlowConfigRepository;
        this.orderItemRepository = orderItemRepository;
    }

    /**
     * 展示申请的或审核过的表单列表
     *
     * @return
     */
    @GetMapping
    @ApiOperation(tags={"CMC","CSC"},value = "流程实例列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult list(Pager pager, String type, @Value(Common.REQ_CONTEXT) RequestContext context) {
        pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
        if (StringUtils.isBlank(type)) {
            return new BocloudResult(false, "重要参数为空");
        }
        pager.getParams().forEach(param -> {
            if (null != param.getParam().get("name") && StringUtils.isNotBlank((String) param.getParam().get("name"))) {
                param.getParam().put("o.name", param.getParam().get("name"));
                param.getParam().remove("name");
                return;
            }
        });
        return engine.query().getGridBeanOfFlowInstances(pager, context, type);
    }

    /**
     * 开始流程
     *
     * @param
     * @return 返回结果
     */
    @PostMapping
    @ApiOperation(tags={"CMC","CSC"},value = "启动流程实例", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @Deprecated
    public BocloudResult start(@RequestParam("defId") Long id, @Value(Common.REQ_CONTEXT) RequestContext context,
                               @RequestParam(value = Common.PARAMS, required = true) String params) {
        JSONObject object = JSONObject.parseObject(params);
        Execution execution = engine.startInstanceById(id, context,null, object.getObject(Common.PARAMS, JSONObject.class));
        if (execution != null) {
            return new BocloudResult(true, execution.getFlowInstance().getId(), "启动流程成功");
        } else {
            return new BocloudResult(false, "启动流程失败");
        }
    }

    /**
     * 撤销流程
     *
     * @return
     */
    @PatchMapping(value = "/{id}")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "撤销流程实例", response = BocloudResult.class, httpMethod = "PATCH", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult patch(@PathVariable(Common.ID) Long id, @RequestBody FlowInstancePatchParam patchParam,
                               @Value(Common.REQ_CONTEXT) RequestContext context) {
        if ("terminate".equals(patchParam.getAction())) {
            boolean success = engine.flowInstance().cancel(id, context.getTarget(), null, true);
            return new BocloudResult(success, success ? "流程撤销成功" : "流程撤销失败");
        } else {
            return ResultTools.formatErrResult();
        }
    }

    /**
     * 流程实例详情
     *
     * @return
     */
    @GetMapping(value = "/{id}")
    @ApiOperation(tags={"CMC","CSC"},value = "流程实例详情", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult detail(@PathVariable(Common.ID) Long id) {
        FlowInstance flowInstance = engine.query().getFlowInstance(id);
        if (flowInstance == null) {
            return new BocloudResult(false, "未查询到数据");
        } else {
            return new BocloudResult(true, flowInstance, "Success!");
        }
    }

    /**
     * 流程实例审核节点
     *
     * @param id
     * @param context
     * @return
     */
    @GetMapping(value = "/{id}/nodes")
    @ApiOperation(tags={"CMC","CSC"},value = "流程实例审核节点列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult nodes(@PathVariable(Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context,
                               @RequestParam(value = "type") String type) {
        if ("audit".equals(type)) {
            List<Param> params = new ArrayList<>();
            params.add(new Param(MapTools.simpleMap("t.instanceId", id), Sign.EQ));
            params.add(new Param(MapTools.simpleMap("instanceState", FlowConstants.STATE_ACTIVE), Sign.EQ));
            params.add(new Param(MapTools.simpleMap("nodeState", FlowConstants.STATE_ACTIVE), Sign.EQ));
            params.add(new Param(MapTools.simpleMap("actorId", context.getTarget()), Sign.EQ));
            params.add(new Param(MapTools.simpleMap("actorCatalog", RequestContext.Catalog.Tenant == context.getCatalog() ? RequestContext.Catalog.Tenant : RequestContext.Catalog.User), Sign.EQ));
            // 获取流程定义id
            FlowInstance flowInstance = engine.query().getFlowInstance(id);
            DefinitionModel definitionModel = engine.definition().getDefinition(flowInstance.getDefId()).getModel();
            Set<RequestContext> userIds = new HashSet<>();
            Map<String, Set<RequestContext>> curNameAndNextActors = new HashMap<>();

            List<FlowNode> list = Lists.newArrayList();
            List<BasicDomainModel<FlowNode, FlowNodeModel>> basicDomainModels = engine.query().getActiveNodes(params);
            for (BasicDomainModel<FlowNode, FlowNodeModel> basicModel : basicDomainModels) {
                FlowNode entity = basicModel.getEntity();
                BaseNodeModel currentNode = definitionModel.getNode(entity.getNodeId());
                if (null == currentNode) {
                    continue;
                }
                List<AuditTaskModel> nextModels = currentNode.getNextModels(AuditTaskModel.class);
                Set<RequestContext> nextActors = new HashSet<>();
                if (!ListTool.isEmpty(nextModels)) {
                    for (AuditTaskModel nextModel : nextModels) {
                        nextActors = assignmentHandler.getAssignees(nextModel, flowInstance.getProjectId(), context);
                    }
                    userIds.addAll(nextActors);
                }
                curNameAndNextActors.put(currentNode.getId(), nextActors);
                list.add(entity);
            }

            // 查询用户名称
            if (userIds != null && userIds.size() > 0 && !ListTool.isEmpty(list)) {
                for (FlowNode flowNode : list) {
                    Set<RequestContext> nextActors = curNameAndNextActors.get(flowNode.getNodeId());
                    StringBuilder names = new StringBuilder();
                    String actorNames = null;
                    for (RequestContext actorContext : nextActors) {
                        String name = flowQueryService.getDisplayName(actorContext);
                        names = names.append(name).append(",");
                    }
                    if (names.length() > 0) {
                        actorNames = names.substring(0, names.lastIndexOf(","));
                    }
                    flowNode.setNextActorNames(actorNames);
                }
            }
            return new BocloudResult(true, list, "Success");
        } else if ("record".equals(type)) {
            FlowInstance flowInstance = engine.query().getFlowInstance(id);
            List<Param> params = new ArrayList<>();
            params.add(new Param(MapTools.simpleMap("t.instanceId", id), Sign.EQ));
            params.add(new Param(MapTools.simpleMap("t.gmtFinish", "1970-1-1"), Sign.GT));

            List<FlowNode> nodes;
            // 判断用户是否创建此流程
            Map<String, String> sorter = new LinkedHashMap<>();
            sorter.put("gmtCreate", Common.ONE);
            sorter.put("gmtFinish", Common.ONE);
            sorter.put(Common.ID, Common.ONE);
            if (flowInstance.getCreatorId().equals(context.getTarget())) {
                nodes = engine.query().getNodes(context, new Pager(1, Integer.MAX_VALUE, params, sorter));
            } else {
                // 用户是否参与此流程
                params.add(new Param(MapTools.simpleMap("actorId", context.getTarget()), Sign.EQ));
                // 查询有当前参与者的节点
                nodes = engine.query().getNodes(context, new Pager(1, Integer.MAX_VALUE, params, sorter));
                // 如果不为空，说明有参与，查询所有的节点
                if (nodes != null) {
                    params = new ArrayList<>();
                    params.add(new Param(MapTools.simpleMap("t.instanceId", id), Sign.EQ));
                    params.add(new Param(MapTools.simpleMap("t.gmtFinish", "1970-1-1"), Sign.GT));
                    nodes = engine.query().getNodes(context, new Pager(1, Integer.MAX_VALUE, params, sorter));
                }
            }
            if (!ListTool.isEmpty(nodes)) {
                for (FlowNode node : nodes) {
                    if (FlowConstants.STATE_ACTIVE.equals(node.getNodeState())) {
                        node.setVariable(null);
                        if (FlowConstants.STATE_CANCEL.equals(flowInstance.getInstanceState())) {
                            // 给前端用的，前端希望知道活动中的node所在流程是否撤销
                            node.setNodeState(FlowConstants.STATE_CANCEL);
                        }
                    } else if (FlowConstants.STATE_FINISH.equals(node.getNodeState())) {
                        // 给前端用的，前端希望知道node是否审核通过,好做展示
                        node.setNodeState((String) node.getVariableMap().get(FlowConstants.ARGS_AUDIT_FLAG));
                    }
                    if (StringUtils.isNotBlank(node.getGroupId())) {
                        List<Attachment> attachments = attachmentService.list(node.getGroupId());
                        node.setAttachments(attachments);
                    }
                }
                return new BocloudResult(true, nodes, "Success");
            }
            return new BocloudResult(false, "未查询到数据");
        } else {
            return ResultTools.formatErrResult();
        }
    }

}
