package com.bocloud.cos.booter.utils;

import net.sf.jasperreports.engine.*;
import net.sf.jasperreports.engine.data.JRBeanCollectionDataSource;
import net.sf.jasperreports.engine.design.*;
import net.sf.jasperreports.engine.xml.JRXmlLoader;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;

/**
 * @author hufy 处理报表相关工具类 JasperCompileManager 编译管理器 JasperFillManager 填充管理器
 * JRXmlLoader xml加载器 JasperPrintManager 打印管理器 JasperExportManager 导出管理器
 */
public class ReportToolUtils {
    private static Logger logger = LoggerFactory.getLogger(ReportToolUtils.class);

    /**
     * 填充报表基础方法
     *
     * @param reportPath 报表路径 .jasper格式文件
     * @param list       传入数据
     * @return
     */
    public static JasperPrint fillReport(String reportPath, List<Object> list) {
        // 设置数据源
        JRBeanCollectionDataSource beanColDataSource = new JRBeanCollectionDataSource(list);
        Map<String, Object> map = new HashMap<String, Object>(16);
        JasperPrint jasperPrint = null;
        try {
            // 填充报表
            jasperPrint = JasperFillManager.fillReport(reportPath, map, beanColDataSource);
        } catch (JRException e) {
            logger.info("初始化报表失败：" + e.getStackTrace());
        }
        return jasperPrint;
    }

    /**
     * 动态报表
     *
     * @param reportPath 报表路径 .jrxml 格式文件
     * @param list       传入数据
     * @param column     动态列
     * @return
     */
    public static JasperPrint dynFileReport(String reportPath, List<Object> list, List<String> column) {
        JasperDesign jdesign = null;
        try {
            // 获得报表设计器
            jdesign = JRXmlLoader.load(reportPath);
        } catch (JRException e) {
            logger.info("获得报表设计器失败：" + e.getStackTrace());
        }
        if (jdesign == null) {
            return null;
        }
        // 根据传入的列动态设计类
        ReportToolUtils.handleColumn(jdesign, column);
        // 设置报表数据源
        JRBeanCollectionDataSource beanColDataSource = new JRBeanCollectionDataSource(list);
        // 报表参数
        Map<String, Object> map = new HashMap<String, Object>(16);
        JasperPrint jasperPrint = null;
        try {
            // 编译报表
            JasperReport jreport = JasperCompileManager.compileReport(jdesign);
            // 获得报表输出
            jasperPrint = JasperFillManager.fillReport(jreport, map, beanColDataSource);
        } catch (JRException e) {
            logger.error(ExceptionUtils.getFullStackTrace(e));
        }
        return jasperPrint;
    }

    /**
     * 根据传入的列动态的生成报表列
     *
     * @param jdesign
     * @param column
     */
    public static void handleColumn(JasperDesign jdesign, List<String> column) {
        if (column != null) {
            JRDesignBand cHeader = (JRDesignBand) jdesign.getColumnHeader();
            JRBand cDetailBand = jdesign.getDetailSection().getBands()[0];
            JRDesignBand cDetail = null;
            if (cDetailBand != null && cDetailBand instanceof JRDesignBand) {
                cDetail = (JRDesignBand) cDetailBand;
            }
            // 获得报表头部元素
            JRElement[] esHeader = cHeader.getElements();
            // 获得报表数据部分列元素
            JRElement[] esDetail = cDetail.getElements();
            List<JRDesignStaticText> headerList = new ArrayList<JRDesignStaticText>();
            for (JRElement jrElement : esHeader) {
                if (jrElement instanceof JRDesignStaticText) {
                    headerList.add((JRDesignStaticText) jrElement);
                }
            }
            // 对列按照列的x值进行排序
            sortHeader(headerList);
            List<JRDesignTextField> detailList = new ArrayList<JRDesignTextField>();
            for (JRElement jrElement : esDetail) {
                if (jrElement instanceof JRDesignTextField) {
                    detailList.add((JRDesignTextField) jrElement);
                }
            }
            // 对列按照列的x值进行排序
            sortDetail(detailList);
            // 获得删除的所有比例
            int delLength = 0;
            StringBuffer lengthBuffer = new StringBuffer();
            int userLength = 0;
            // 获得所有列占的宽度 -- 直接使用jdesign.getPageWidth() 获得的是页面的宽度，页面可能存在空白区域，造成拼接的页面数据不准
            int lengthAll = getAllWidth(esHeader);
            for (int i = 0; i < headerList.size(); i++) {
                JRDesignElement e = (JRDesignElement) headerList.get(i);
                String v = "";
                if (e instanceof JRStaticText) {
                    JRStaticText text = (JRStaticText) e;
                    v = text.getText();
                }
                // header元素不在传入的列中
                if (!column.contains(v)) {
                    for (int j = 0; j < detailList.size(); j++) {
                        JRDesignElement ee = (JRDesignElement) detailList.get(i);
                        // 删除detail中所有列x一样的
                        if (ee.getX() == e.getX()) {
                            cDetail.removeElement(ee);
                        }
                    }
                    //防止分母为0
                    if (0 == lengthAll) {
                        lengthAll = 1;
                    }
                    // 删除头
                    cHeader.removeElement(e);
                    // 获得删除的所有比例
                    if (i == 0) {
                        delLength += headerList.get(0).getX() / lengthAll;
                    } else if (i == headerList.size() - 1) {
                        delLength += jdesign.getPageWidth() - e.getX() / lengthAll;
                    } else {
                        delLength += headerList.get(i + 1).getX() - e.getX() / lengthAll;
                    }
                } else {// 获得剩余的所有长度
                    if (i == 0) {
                        lengthBuffer.append(headerList.get(1).getX());
                        userLength += headerList.get(1).getX();
                    } else if (i == headerList.size() - 1) {
                        lengthBuffer.append(jdesign.getPageWidth() - e.getX());
                        userLength += jdesign.getPageWidth() - e.getX();
                    } else {
                        lengthBuffer.append(headerList.get(i + 1).getX() - e.getX());
                        userLength += headerList.get(i + 1).getX() - e.getX();
                    }
                    lengthBuffer.append(",");
                }
            }
            if (delLength == 1) {
                logger.info("报表显示列宽度为0");
            } else {
                JRElement[] esheader = cHeader.getElements();
                String[] intHeader = lengthBuffer.toString().split(",");
                //防止分母为0
                if (userLength == 0) {
                    userLength = 1;
                }
                for (int i = 0; i < esheader.length; i++) {
                    if (i == 0) {
                        cHeader.getElements()[i].setX(0);
                        cDetail.getElements()[i].setX(0);
                    } else {
                        cHeader.getElements()[i].setX(cHeader.getElements()[i - 1].getX()
                                + Integer.parseInt(intHeader[i - 1]) * lengthAll / userLength);
                        cDetail.getElements()[i].setX(cDetail.getElements()[i - 1].getX()
                                + Integer.parseInt(intHeader[i - 1]) * lengthAll / userLength);
                    }
                    // 调整列所占宽度
                    cHeader.getElements()[i].setWidth(cHeader.getElements()[i].getWidth() * lengthAll / userLength);
                    cDetail.getElements()[i].setWidth(cDetail.getElements()[i].getWidth() * lengthAll / userLength);
                }
            }
        }
    }

    /**
     * 获得所有列占的总宽度
     *
     * @param esHeader
     * @return
     */
    private static int getAllWidth(JRElement[] esHeader) {
        int allWidth = 0;
        for (JRElement jrElement : esHeader) {
            allWidth = allWidth + jrElement.getWidth();
        }
        return allWidth;
    }

    /**
     * 对报表列头部进行排序
     *
     * @param headerList
     */
    public static void sortHeader(List<JRDesignStaticText> headerList) {
        Collections.sort(headerList, Comparator.comparing(arg0 -> new Integer(arg0.getX())));
    }

    /**
     * 对报表列数据进行排序
     *
     * @param detailList
     */
    public static void sortDetail(List<JRDesignTextField> detailList) {
        Collections.sort(detailList, Comparator.comparing(arg0 -> new Integer(arg0.getX())));
    }
}
