package com.bocloud.cos.entity;

import com.bocloud.common.utils.DateSerializer;
import com.bocloud.entity.bean.Generic;
import com.bocloud.entity.annotations.*;
import com.bocloud.entity.meta.PKStrategy;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;

/**
 * 报表配置模板
 *
 * @author lyy
 * @version 2.0
 * @time 2017年12月21日
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
@EqualsAndHashCode(callSuper = true)
@Table("rp_report_config_template")
public class ReportConfigTemplate extends Generic {

    @PK(value = PKStrategy.AUTO)
    private Long id;
    /**
     * 模板名称
     */
    @Column("name")
    private String name;
    /**
     * 是否启用
     */
    @Column("status")
    private Boolean status;
    /**
     * 周报类型
     */
    @Column("cycle_type")
    private String cycleType;
    /**
     * 开始时间
     */
    @Column("start_time")
    private String startTime;
    /**
     * 结束时间
     */
    @Column("end_time")
    private String endTime;
    /**
     * 开始日期
     */
    @Column("start_date")
    private String startDate;
    /**
     * 结束日期
     */
    @Column("end_date")
    private String endDate;
    /**
     * 选择的星期
     */
    @Column("weeks")
    private String weeks;
    /**
     * 生成时间
     */
    @Column("build_time")
    private String buildTime;
    /**
     * 创建人id
     */
    @Column("create_id")
    private Long createId;
    /**
     * 修改人id
     */
    @Column("mender_id")
    private Long menderId;
    /**
     * 创建时间
     */
    @Column("gmt_create")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtCreate;
    /**
     * 修改时间
     */
    @Column("gmt_modify")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtModify;
    /**
     * 是否删除
     */
    @Column("is_deleted")
    private Boolean deleted;
    /**
     * 报表类别
     */
    @Column("report_category")
    private String reportCategory;
    /**
     * 租户id
     */
    @Column("tenant_id")
    private Long tenantId;

    /**
     * 创建人
     */
    @IgnoreInsert
    @IgnoreUpdate
    @Column("creater")
    private String creater;
    /**
     * 订阅状态
     */
    @IgnoreInsert
    @IgnoreUpdate
    @Column("subscriptionStatus")
    private Boolean subscriptionStatus;

    /**
     * 有效起始日期
     */
    @Column("begin_date")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    private Date beginDate;

    /**
     * 有效终止日期
     */
    @Column("finish_date")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    private Date finishDate;

    public ReportConfigTemplate(String name, Boolean status, String cycleType, String startTime, String endTime,
                                String startDate, String endDate, String weeks, String buildTime, Long createId, Long menderId,
                                Date gmtCreate, Date gmtModify, Boolean deleted, String reportCategory, Long tenantId, String creater,
                                Boolean subscriptionStatus) {
        super();
        this.name = name;
        this.status = status;
        this.cycleType = cycleType;
        this.startTime = startTime;
        this.endTime = endTime;
        this.startDate = startDate;
        this.endDate = endDate;
        this.weeks = weeks;
        this.buildTime = buildTime;
        this.createId = createId;
        this.menderId = menderId;
        this.gmtCreate = gmtCreate;
        this.gmtModify = gmtModify;
        this.deleted = deleted;
        this.reportCategory = reportCategory;
        this.tenantId = tenantId;
        this.creater = creater;
        this.subscriptionStatus = subscriptionStatus;
    }
}
