package com.bocloud.cos.model;

import com.bocloud.common.enums.ResourceCategory;
import com.bocloud.common.utils.DateSerializer;
import com.bocloud.cos.enums.OrderStatus;
import com.bocloud.cos.enums.ProductionCatalog;
import com.bocloud.entity.annotations.Column;
import com.bocloud.entity.bean.Generic;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;

/**
 * 计量计费信息报告
 *
 * @author lyy
 * @version 2.0
 * @time 2017年12月28日
 */
public class ChargeInformationReport extends Generic {
    /**
     * 订单号
     */
    @Column("sn")
    private String sn;
    /**
     * 产品名称
     */
    @Column("resource_category")
    private String resourceCategory;
    /**
     * 订单类型
     */
    @Column("category")
    private String category;
    /**
     * 下单时间
     */
    @Column("gmt_create")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtCreate;
    /**
     * 支付时间
     */
    @Column("gmt_paid")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtPaid;
    /**
     * 订单状态
     */
    @Column("status")
    private String status;

    private Long number;

    private String projectName;

    private String userName;

    private String tenantName;

    public String getSn() {
        return sn;
    }

    public void setSn(String sn) {
        this.sn = sn;
    }

    public String getResourceCategory() {
        if (null == resourceCategory) {
            return "";
        }
        ResourceCategory value = null;
        try {
            value = ResourceCategory.valueOf(resourceCategory);
        } catch (IllegalArgumentException e) {
            // 没有这个枚举直接返回名称
            return resourceCategory;
        }
        switch (value) {
            case COMPUTER:
                return "虚拟机购买";
            case STORAGE:
                return "云硬盘购买";
            case ROUTER:
                return "路由器购买";
            case FLOATING_IP:
                return "云IP购买";
            case SNAPSHOT:
                return "云快照购买";
            case IMAGE:
                return "云镜像购买";
            default:
                break;
        }
        return resourceCategory;
    }

    public void setResourceCategory(String resourceCategory) {
        this.resourceCategory = resourceCategory;
    }

    public String getCategory() {
        if (null == category) {
            return "";
        }
        ProductionCatalog catalog = null;
        try {
            catalog = ProductionCatalog.valueOf(category);
        } catch (IllegalArgumentException e) {
            return category;
        }
        switch (catalog) {
            case RESOURCE:
                return "资源类商品";
            case SERVICE:
                return "资源类商品";
            default:
                break;
        }
        return category;
    }

    public void setCategory(String category) {
        this.category = category;
    }

    public Date getGmtCreate() {
        return gmtCreate;
    }

    public void setGmtCreate(Date gmtCreate) {
        this.gmtCreate = gmtCreate;
    }

    public Date getGmtPaid() {
        return gmtPaid;
    }

    public void setGmtPaid(Date gmtPaid) {
        this.gmtPaid = gmtPaid;
    }

    public String getStatus() {
        if (null == status) {
            return "";
        }
        OrderStatus value = null;
        try {
            value = OrderStatus.valueOf(status);
        } catch (IllegalArgumentException e) {
            return status;
        }

        switch (value) {
            case UNREVIEW:
                return "未审核";
            case REVIEWING:
                return "审核中";
            case SUCCESS:
                return "已完成";
            case FAILURE:
                return "失败";
            default:
                break;
        }
        return status;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    public Long getNumber() {
        return number;
    }

    public void setNumber(Long number) {
        this.number = number;
    }

    public String getProjectName() {
        return projectName;
    }

    public void setProjectName(String projectName) {
        this.projectName = projectName;
    }

    public String getUserName() {
        return userName;
    }

    public void setUserName(String userName) {
        this.userName = userName;
    }

    public String getTenantName() {
        return tenantName;
    }

    public void setTenantName(String tenantName) {
        this.tenantName = tenantName;
    }

}
