package com.bocloud.cos.repository;

import com.bocloud.cos.common.utils.ArrayFormatter;
import com.bocloud.cos.common.utils.MathUtils;
import com.bocloud.cos.enums.ReportFrequenceType;
import com.bocloud.cos.model.MonitorInfoReport;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;
import org.apache.commons.lang3.StringUtils;

import java.util.List;
import java.util.Map;

/**
 * @author hufy
 * @since 2018-10-18 modify by sxp 修改为调用CMP接口查询监控数据
 */
@Repository("monitoryReportRepository")
public class MonitorReportRepository extends BasicGenericDao<MonitorInfoReport, Long> {
    public MonitorReportRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }
//    @Autowired
//    InfluxDbDao influxdbDao;

    public List<MonitorInfoReport> queryMonitorInfoReport(String[] resourceIds, ReportFrequenceType reportFrequenceType,
                                                          Map<String, Object> param) throws Exception {
        // 从数据库查询数据
        return getReportFromMysql(resourceIds, param);
    }


    @SuppressWarnings("unused")
    private boolean ifHasData(MonitorInfoReport monitorInfoReport) {
        return !StringUtils.isNotEmpty(monitorInfoReport.getUserCpuUsage())
                && !StringUtils.isNotEmpty(monitorInfoReport.getSysCpuUsage())
                && !StringUtils.isNotEmpty(monitorInfoReport.getRamSummit())
                && !StringUtils.isNotEmpty(monitorInfoReport.getRamUsage());
    }


    public List<MonitorInfoReport> getReportFromMysql(String[] resourceIds, Map<String, Object> param)
            throws Exception {
        List<Long> resourceIdList = ArrayFormatter.toLongList(resourceIds);
        param.put("resourceIds", resourceIdList);
        String sql = "SELECT * FROM cloud_server a WHERE a.is_deleted = 0 AND a.id in (:resourceIds) ";
        return this.list(MonitorInfoReport.class, sql, param);
    }

    /**
     * map 转为 bean
     *
     * @param maps
     * @param monitorInfoReports
     */
    private void converMysqlMapToBean(List<Map<String, Object>> maps, List<MonitorInfoReport> monitorInfoReports) {
        Long number = 0L;
        for (Map<String, Object> map : maps) {
            MonitorInfoReport report = new MonitorInfoReport();
            report.setName((String) map.get("name"));
            report.setVendorId(MathUtils.castToLongClass(map.get("vendor_id")));
            report.setNumber(number++);
            report.setInstanceId((String) map.get("instance_id"));
            report.setManagerIp((String) map.get("manager_ip"));
            report.setVendorType((String) map.get("vendor_type"));
            report.setOsCategory((String) map.get("os_category"));
            report.setUuid((String) map.get("uuid"));
            report.setTarget((Long) map.get("target"));
            monitorInfoReports.add(report);
        }
    }
}
