package com.bocloud.cos.repository;

import com.bocloud.common.model.Param;
import com.bocloud.cos.entity.PcTemplateProperty;
import com.bocloud.database.core.BasicDao;
import com.bocloud.database.utils.PageHelper;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 公有云账单
 *
 * @author lyy
 * @version 1.0
 * @date 2021/2/26 17:28
 */
@Repository("pcBillsRepository")
public class PcBillsRepository extends BasicDao {

    public int count(List<Param> params, Map<Long, List<PcTemplateProperty>> propertyMap, String type) throws Exception {
        String sql = buildSql(0, 0, params, null, propertyMap, false, false, type);
        sql = "select count(1) from (" + sql + ") B";
        return this.countQuery(sql, null).intValue();
    }

    public List<Map<String, Object>> list(int page, int rows, List<Param> params, Map<String, String> sorter,
                                          Map<Long, List<PcTemplateProperty>> propertyMap, Boolean simple, Boolean isPropertyAll, String type) throws Exception {
        String sql = buildSql(page, rows, params, sorter, propertyMap, simple, isPropertyAll, type);
        List<Map<String, Object>> list = this.queryForList(sql);
        if (list == null) {
            list = new ArrayList<>();
        }
        return list;
    }

    private String buildSql(int page, int rows, List<Param> params, Map<String, String> sorter,
                            Map<Long, List<PcTemplateProperty>> propertyMap, Boolean simple, Boolean isPropertiesAll, String type) throws Exception {
        String resultSql = "SELECT * FROM (";
        StringBuilder sqlBuilder = new StringBuilder();
        for (Long id : propertyMap.keySet()) {
            // 行转列的表头
            String sql = "SELECT GROUP_CONCAT('MAX(IF(PTP.ID = ',PTP.id,',PTPV.PROPERTY_VALUE,NULL)) AS ''',PTP.`name`,'''') as sqlResult" +
                    " from pc_template_property AS PTP  where template_id = " + id;
            String sqlResult;
            // 只查询选中的列
            if (!isPropertiesAll) {
                StringBuilder whereSql = new StringBuilder();
                whereSql.append(" and id in(");
                List<PcTemplateProperty> properties = propertyMap.get(id);
                for (PcTemplateProperty property : properties) {
                    whereSql.append(property.getId()).append(",");
                }
                sql += whereSql.substring(0, whereSql.length() - 1) + ")";
                List<Map<String, Object>> result = this.queryForList(sql);
                if (result == null || result.isEmpty()) {
                    throw new Exception("查询标题数据失败");
                }
                sqlResult = result.get(0).get("sqlResult").toString();
            } else {
                sqlResult = "";
            }
            // 拼接sql
            sqlBuilder.append(" SELECT ins.group_id AS id, ins.gmt_create");
            if (sqlResult != null && !"".equals(sqlResult)) {
                sqlBuilder.append(",");
                sqlBuilder.append(sqlResult);
            }
            sqlBuilder.append(" FROM (");
            sqlBuilder.append(" select a.gmt_create, a.template_id, a.group_id, a.id, a.property_id, a.is_deleted from pc_template_property_value a where id in ( select min(id) from pc_template_property_value group by group_id) and is_deleted = 0 ");
            sqlBuilder.append(") ins");
            sqlBuilder.append(" LEFT JOIN pc_template_property_value AS PTPV ON ins.group_id = PTPV.group_id ");
            sqlBuilder.append(
                    "LEFT JOIN pc_template_property AS PTP ON PTPV.property_id = PTP.id where 1=1 and ins.is_deleted=0 ");
            sqlBuilder.append(" and ins.template_id = " + id);
            sqlBuilder.append(" GROUP BY ins.id UNION");
        }
        resultSql += StringUtils.substringBeforeLast(sqlBuilder.toString(), "UNION") + " )a";
        String account = getField(params, "account");
        String productType = getField(params, "productType");
        String product = getField(params, "product");
        String billType = getField(params, "billType");
        StringBuilder whereSql = new StringBuilder();
        if (null == account && null == productType && null == product && null == billType) {
            whereSql.append(" order by a.gmt_create desc");
        } else {
            whereSql.append(" HAVING ");
            StringBuilder search = new StringBuilder();
            if (null != account) {
                search.append("and 账号 = " + "'" + account + "'");
            }
            if (null != productType) {
                if ("ALI".equalsIgnoreCase(type)) {
                    search.append("and 产品明细 = " + "'" + productType + "'");
                } else if ("HUAWEI".equalsIgnoreCase(type)) {
                    search.append("and 产品类型 = " + "'" + productType + "'");
                }
            }
            if (null != product) {
                search.append("and 产品 = " + "'" + product + "'");
            }
            if (null != billType) {
                search.append("and 账单类型 = " + "'" + billType + "'");
            }
            if (null != search.toString() && !"".equals(search.toString())) {
                String searchSql = StringUtils.substringAfter(search.toString(), " ");
                whereSql.append(searchSql);
            }
            whereSql.append(" order by a.gmt_create desc");
        }
        resultSql += whereSql.toString();
        if (simple == null || simple.booleanValue() == false) {
            if (page != 0) {
                resultSql += PageHelper.buildPageSql(page, rows);
            }
        }
        return resultSql;
    }

    public String getField(List<Param> params, String field) {
        if ((null != params) && (!params.isEmpty())) {
            for (Param param : params) {
                for (String key : param.getParam().keySet()) {
                    if (field.equals(key)) {
                        return param.getParam().get(key).toString();
                    }
                }
            }
        }
        return null;
    }
}