package com.bocloud.cos.repository;

import com.bocloud.cos.common.utils.ArrayFormatter;
import com.bocloud.cos.enums.ReportFrequenceType;
import com.bocloud.cos.model.GraphData;
import com.bocloud.cos.model.GraphSeries;
import com.bocloud.cos.model.PlatformInfoReport;
import com.bocloud.database.core.BasicDao;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import org.apache.curator.shaded.com.google.common.collect.Maps;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.*;

/**
 * @author hufy
 */
@Repository("platformInfoReportRepository")
public class PlatformInfoReportRepository extends BasicGenericDao<PlatformInfoReport, Long> {

    @Autowired
    private BasicDao basicDao;

    public PlatformInfoReportRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    private List<PlatformInfoReport> convertMapToPlatformInfoReport(List<Map<String, Object>> list) {
        List<PlatformInfoReport> reportList = new ArrayList<>();
        Long number = 1L;
        for (Map<String, Object> map : list) {
            PlatformInfoReport platformInfoReport = new PlatformInfoReport();
            platformInfoReport.setCloudVendorId((Long) map.get("id"));
            platformInfoReport.setCloudVendorName((String) map.get("name"));
            platformInfoReport.setServerCount((Long) map.get("serverCount"));
            platformInfoReport.setCloudServerCount((Long) map.get("cloudServerCount"));
            platformInfoReport.setResourceAlarmCount((Long) map.get("resourceAlarmCount"));
            platformInfoReport.setTenantCount((Long) map.get("tenantCount"));
            platformInfoReport.setServerNumber((Long) map.get("serverNumber"));
            platformInfoReport.setCloudServerNumber((Long) map.get("cloudServerNumber"));
            platformInfoReport.setResourceAlarmNumber((Long) map.get("resourceAlarmNumber"));
            if (reportList.size() == 0) {
                platformInfoReport.setNumber(number);
            } else {
                number++;
                platformInfoReport.setNumber(number);
            }
            reportList.add(platformInfoReport);
        }
        return reportList;
    }

    @SuppressWarnings("unused")
    private List<GraphData> convertMapToLinechartData(List<Map<String, Object>> list) {
        List<GraphData> graphDatas = new ArrayList<>();
        Set<String> timeSet = new HashSet<>();
        // key = name + , +time value = num
        Map<String, Long> values = new HashMap<>();
        // key = name value = graphData
        Map<String, GraphData> graphDataMap = new HashMap<>();
        // 处理时间
        for (Map<String, Object> iteraMap : list) {
            String vendorName = (String) iteraMap.get("name");
            String time = (String) iteraMap.get("time");
            Long value = (Long) iteraMap.get("cloudServerCount");
            timeSet.add(time);
            values.put(vendorName + "," + time, value);

        }
        List<String> timeList = new ArrayList<>(timeSet);
        Collections.sort(timeList);
        for (Map<String, Object> iteraMap : list) {
            String vendorName = (String) iteraMap.get("name");
            if (!graphDataMap.containsKey(vendorName)) {
                GraphData graphData = new GraphData();
                graphData.setName(vendorName);
                graphData.setTimes(timeList);
                List<GraphSeries> graphSeriesList = new ArrayList<>();
                GraphSeries graphSeries = new GraphSeries();
                graphSeries.setData(new ArrayList<>());
                graphSeries.setName(vendorName);
                graphSeriesList.add(graphSeries);
                graphData.setSeries(graphSeriesList);
                graphDataMap.put(vendorName, graphData);
            }
        }

        for (String time : timeList) {
            Set<Map.Entry<String, GraphData>> entries = graphDataMap.entrySet();
            for (Map.Entry<String, GraphData> entry : entries) {
                String vendorName = entry.getKey();
                GraphData graphData = entry.getValue();
                graphData.getSeries().get(0).getData().add(nullToZero(values.get(vendorName + "," + time)));
            }
        }

        for (Map.Entry<String, GraphData> graphDataEntry : graphDataMap.entrySet()) {
            graphDatas.add(graphDataEntry.getValue());
        }
        return graphDatas;
    }

    private Long nullToZero(Object o) {
        if (o == null) {
            return 0L;
        }
        return (Long) o;
    }

    public List<GraphData> queryVmLineChartByTime(ReportFrequenceType frequenceType, Map<String, Object> params,
                                                  String[] resourceIds) throws Exception {
        params.put("ids", ArrayFormatter.toLongList(resourceIds));
        String timeClause = "";
        String dateFormatClause = "";
        switch (frequenceType) {
            case ByDay:
                timeClause = " and gmt_create >=:startTime and gmt_create <= :endTime ";
                dateFormatClause = " DATE_FORMAT(gmt_create,'%Y-%m-%d %H') ";
                break;
            case ByWeek:
                timeClause = " and DATE_FORMAT(gmt_create,'%Y-%m-%d') in (:dayList) ";
                dateFormatClause = " DATE_FORMAT(gmt_create,'%Y-%m-%d') ";
                break;
            case ByMonth:
                timeClause = " and gmt_create >=:startDate and gmt_create< :endDate ";
                dateFormatClause = " DATE_FORMAT(gmt_create,'%Y-%m-%d') ";
                break;
            default:
                break;
        }
        String sql = "SELECT cv.`name`, cv.id, IFNULL(time,'') AS time, ifnull(cs.cloudServerCount, 0) AS cloudServerCount FROM cloud_vendor cv "
                + "LEFT JOIN ( SELECT vendor_id, " + dateFormatClause
                + " as time , count(*) AS cloudServerCount FROM cloud_server " + "WHERE is_deleted = 0 and is_template = 0 " + timeClause
                + "GROUP BY vendor_id," + dateFormatClause + ") cs " + "ON cv.id = cs.vendor_id"
                + " where cv.id in (:ids)";
        List<Map<String, Object>> maps = this.basicDao.queryForList(sql, params);
        return convertMapToLinechartData(maps);
    }

    public List<PlatformInfoReport> queryPlatformTableInfo(String[] resourceIds, ReportFrequenceType reportFrequenceType, Map<String, Object> param) throws Exception {
        param.put("ids", ArrayFormatter.toLongList(resourceIds));
        String timeClause = "";
        // String tenantTimeClause = "";
        Map<String, Object> params = Maps.newHashMap();
        switch (reportFrequenceType) {
            case ByDay:
                timeClause = " and gmt_create >=:startTime and gmt_create <= :endTime ";
                // tenantTimeClause = " and p.gmt_create >=:startTime and p.gmt_create <=
                // :endTime ";
                params.put("startTime", param.get("startTime"));
                params.put("endTime", param.get("endTime"));
                break;
            case ByWeek:
                timeClause = " and DATE_FORMAT(gmt_create,'%Y-%m-%d') in (:dayList) ";
                // tenantTimeClause = " and DATE_FORMAT(p.gmt_create,'%Y-%m-%d') in (:dayList)
                // ";
                params.put("dayList", param.get("dayList"));
                break;
            case ByMonth:
                timeClause = " and gmt_create >=:startDate and gmt_create< :endDate ";
                // tenantTimeClause = " and p.gmt_create >=:startDate and p.gmt_create< :endDate
                // ";
                params.put("startDate", param.get("startDate"));
                params.put("endDate", param.get("endDate"));
                break;
            default:
                break;
        }
        String queryTenantSql = " SELECT COUNT(DISTINCT p.tenant_id) as tenant_count FROM project p LEFT JOIN (SELECT DISTINCT bp.business_id FROM business_pool bp LEFT JOIN (SELECT a.* FROM cloud_vendor cv LEFT JOIN (SELECT * FROM pool_group pg WHERE pg.is_deleted = 0 GROUP BY pg.id ) a ON cv.id = a.vendor_id WHERE cv.id = :vendorId AND cv.is_deleted = 0 ) c ON bp.pool_group_id = c.id WHERE bp.pool_group_id = c.id ) d ON p.business_id = d.business_id WHERE p.is_deleted = 0 AND p.business_id = d.business_id AND tenant_id != 0 ";
        // + tenantTimeClause;
        String sql = "SELECT cv.`name`,cv.id,ifnull(s.serverCount,0) as serverCount, "
                + " ifnull(cs.cloudServerCount,0) as cloudServerCount,ifnull(a.resourceAlarmCount,0) as resourceAlarmCount, ifnull(sn.serverNumber, 0) AS serverNumber,ifnull(csn.cloudServerNumber, 0) AS cloudServerNumber,ifnull(an.resourceAlarmNumber, 0) AS resourceAlarmCount "
                + " FROM " + "  cloud_vendor cv LEFT JOIN ( SELECT vendor_id, "
                + "    count(*) AS serverCount   FROM    `server` " + // 物理机
                "  WHERE  is_deleted = 0 " + timeClause + // 时间条件
                "  GROUP BY  vendor_id) s ON cv.id = s.vendor_id "
                + " LEFT JOIN (SELECT vendor_id, count(*) AS serverNumber FROM `server` WHERE is_deleted = 0 GROUP BY vendor_id) sn ON cv.id = sn.vendor_id "
                + " LEFT JOIN ( SELECT " + "    vendor_id, count(*) AS cloudServerCount FROM cloud_server " + // 虚拟机
                "  WHERE  is_deleted = 0 AND is_template = 0 " + timeClause // 时间条件
                + "  GROUP BY  vendor_id ) cs ON cv.id = cs.vendor_id "
                + " LEFT JOIN (SELECT vendor_id, count(*) AS cloudServerNumber FROM cloud_server WHERE is_deleted = 0 AND is_template = 0 GROUP BY vendor_id) csn ON cv.id = csn.vendor_id "
                + " LEFT JOIN (SELECT vendor_id, count(*) AS resourceAlarmNumber FROM alarm a WHERE is_deleted = 0 GROUP BY vendor_id) an ON an.vendor_id = cv.id "
                + " LEFT JOIN (SELECT " + "    vendor_id, count(*) AS resourceAlarmCount " + // 资源告警
                "  FROM alarm a WHERE is_deleted = 0 " + timeClause + // 时间条件
                "  GROUP BY vendor_id) a on a.vendor_id=cv.id " + " where cv.id in (:ids)";
        List<Map<String, Object>> maps = this.basicDao.queryForList(sql, param);
        for (Map<String, Object> map : maps) {
            Long vendorId = (Long) map.get("id");
            params.put("vendorId", vendorId);
            List<Map<String, Object>> lists = this.basicDao.queryForList(queryTenantSql, params);
            for (Map<String, Object> Project : lists) {
                map.put("tenantCount", Project.get("tenant_count"));
            }
        }
        return convertMapToPlatformInfoReport(maps);
    }


}
