package com.bocloud.cos.repository;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.Sign;
import com.bocloud.database.core.BasicDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.internel.entity.CloudServer;
import com.bocloud.internel.entity.User;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * 报表数据查询
 *
 * @author lyy
 * @version 1.0
 * @date 2021/3/11 16:45
 */
@Repository("reportRepository")
public class ReportRepository extends BasicDao {

    public Map<Long, User> userList() throws Exception {
        //缓存所有的用户信息
        String userSql = "SELECT * FROM `user` WHERE is_deleted = 0";
        List<Object> userList = this.queryForList(userSql, User.class);
        Map<Long, User> userMap = Maps.newHashMap();
        for (Object object : userList) {
            User user = (User) object;
            userMap.put(user.getId(), user);
        }
        return userMap;
    }

    public List<CloudServer> vmList(int page, int rows, List<Param> params, Map<String, String> sorter) throws Exception {
        Map<Long, User> userMap = userList();
        //虚拟机信息查询
        StringBuilder sql = new StringBuilder("SELECT a.*, IF(a.catalog = 'User',d.`name`,NULL) AS target_name, b.`name` AS tenant_name, c.`name` AS project_name, e.`name` AS vendor_name, f.`name` AS owner_name FROM cloud_server a LEFT JOIN tenant b ON a.tenant_id = b.id LEFT JOIN project c ON a.project_id = c.id LEFT JOIN `user` d ON a.creator_id = d.id LEFT JOIN cloud_vendor e ON a.vendor_id = e.id LEFT JOIN `user` f ON a.owner_id = f.id WHERE a.is_template = 0 AND a.is_deleted = 0 ");
        for (Param param : params) {
            Sign sign = param.getSign();
            if ("like".equals(sign.getSign())) {
                Map<String, Object> map = param.getParam();
                for (String key : map.keySet()) {
                    String value = String.valueOf(map.get(key));
                    switch (key) {
                        case "tenantName":
                            sql.append(" and b.name like '%").append(value).append("%'");
                            break;
                        case "projectName":
                            sql.append(" and c.name like '%").append(value).append("%'");
                            break;
                        case "targetName":
                            sql.append(" and d.name like '%").append(value).append("%' and a.catalog = 'User'");
                            break;
                        case "ownerName":
                            sql.append(" and f.name like '%").append(value).append("%'");
                            break;
                        default:
                            break;
                    }
                }
            }
        }
        List<CloudServer> vms = Lists.newArrayList();
        sql = new StringBuilder(QueryBuilder.buildRaw(sql.toString(), new Pager(page, rows, null, sorter), ""));
        List<Object> list = this.queryForList(sql.toString(), CloudServer.class);
        for (Object object : list) {
            vms.add((CloudServer) object);
        }
        return vms;

//        List<CloudServer> vms = Lists.newArrayList();
//        if (null == patorsName) {
//            sql = new StringBuilder(QueryBuilder.buildRaw(sql.toString(), new Pager(page, rows, null, sorter), ""));
//            List<Object> list = this.queryForList(sql.toString(), CloudServer.class);
//            for (Object object : list) {
//                CloudServer cloudServer = (CloudServer) object;
//                if (null != cloudServer.getParticipators() && !"".equals(cloudServer.getParticipators())) {
//                    String participators = cloudServer.getParticipators();
//                    String[] userIds = participators.split(",");
//                    List<String> patorsNames = Lists.newArrayList();
//                    for (String userId : userIds) {
//                        patorsNames.add(null != userMap.get(Long.valueOf(userId)) ? userMap.get(Long.valueOf(userId)).getName() : "");
//                    }
//                    cloudServer.setPatorsNames(patorsNames);
//                }
//                vms.add(cloudServer);
//            }
//        } else {
//            List<Object> list = this.queryForList(sql.toString(), CloudServer.class);
//            for (Object object : list) {
//                CloudServer cloudServer = (CloudServer) object;
//                if (null != cloudServer.getParticipators() && !"".equals(cloudServer.getParticipators())) {
//                    String participators = cloudServer.getParticipators();
//                    String[] userIds = participators.split(",");
//                    List<String> patorsNames = Lists.newArrayList();
//                    boolean isShow = false;
//                    for (String userId : userIds) {
//                        patorsNames.add(null != userMap.get(Long.valueOf(userId)) ? userMap.get(Long.valueOf(userId)).getName() : "");
//                        if (null != userMap.get(Long.valueOf(userId))) {
//                            String name = userMap.get(Long.valueOf(userId)).getName();
//                            if (name.contains(patorsName)) {
//                                isShow = true;
//                            }
//                        }
//                    }
//                    if (isShow) {
//                        cloudServer.setPatorsNames(patorsNames);
//                        vms.add(cloudServer);
//                    }
//                }
//            }
//        }
//        return vms;
    }

    public List<CloudServer> list() throws Exception {
        String sql = "SELECT a.*,b.`name` AS host FROM cloud_server a LEFT JOIN `server` b ON a.host_id = b.id WHERE a.is_deleted = 0 AND a.is_template = 0 AND a.participators IS NOT NULL ";
        List<Object> list = this.queryForList(sql, CloudServer.class);
        List<CloudServer> vms = Lists.newArrayList();
        for (Object object : list) {
            vms.add((CloudServer) object);
        }
        return vms;
    }
}