package com.bocloud.cos.repository;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.entity.ServiceBill;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 账单数据访问
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年3月10日
 */
@Repository("serviceBillRepository") public class ServiceBillRepository extends BasicGenericDao<ServiceBill, Long> {

    public ServiceBillRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public List<ServiceBill> list(int page, int rows, Map<String, String> sorter, List<Param> params,
        String serviceCode, String orderSn, List<Long> projectIds) throws Exception {
        String sql =
            "select a.*,b.charge_mode,b.sn order_sn,c.name tenant_name from service_bill a left join service_order b on b.id = a.order_id left join tenant c on c.id = a.tenant_id where 1 = 1 ";
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        if (StringUtils.isNotEmpty(serviceCode)) {
            sql += " and b.service_code = :serviceCode";
            paramMap.put("serviceCode", serviceCode);
        }
        if (StringUtils.isNotEmpty(orderSn)) {
            sql += " and b.sn like '%" + orderSn + "%'";
        }
        if (!ListTool.isEmpty(projectIds)) {
            sql += " and a.project_id in (:projectIds) ";
            paramMap.put("projectIds", projectIds);
        }
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        return this.list(ServiceBill.class, sql, paramMap);
    }

    public int count(List<Param> params, String serviceCode, String orderSn, List<Long> projectIds) throws Exception {
        String sql =
            "select count(a.id) from service_bill a left join service_order b on b.id = a.order_id left join tenant c on c.id = a.tenant_id where 1 = 1 ";
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        if (StringUtils.isNotEmpty(serviceCode)) {
            sql += " and b.service_code = :serviceCode";
            paramMap.put("serviceCode", serviceCode);
        }
        if (StringUtils.isNotEmpty(orderSn)) {
            sql += " and b.sn like '%" + orderSn + "%'";
        }
        if (!ListTool.isEmpty(projectIds)) {
            sql += " and a.project_id in (:projectIds) ";
            paramMap.put("projectIds", projectIds);
        }
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        return this.countQuery(sql, paramMap).intValue();
    }

    public List<ServiceBill> list(int page, int rows, Map<String, String> sorter, List<Param> params,
        String serviceCode, String orderSn, Long userId, List<Long> projectIds) throws Exception {
        String subSql =
            "select a.*,b.charge_mode,b.sn order_sn,c.name tenant_name from service_bill a left join service_order b on b.id = a.order_id left join tenant c on c.id = a.tenant_id where 1 = 1 ";
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        if (StringUtils.isNotEmpty(serviceCode)) {
            subSql += " and b.service_code = :serviceCode";
            paramMap.put("serviceCode", serviceCode);
        }
        if (StringUtils.isNotEmpty(orderSn)) {
            subSql += " and b.sn like '%" + orderSn + "%'";
        }
        subSql = QueryBuilder.buildRaw(subSql, params, null, "a");
        String sql = "select distinct d.* from (" + subSql + ") d  where ( 1=-1 ";
        if (null != userId && 0L != userId) {
            sql += " or d.user_id =:userId ";
            paramMap.put("userId", userId);
        }
        if (null != projectIds && !projectIds.isEmpty()) {
            sql += " or d.project_id in (:projectIds) ";
            paramMap.put("projectIds", projectIds);
        }
        sql += " )";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, null, sorter), "d");
        return this.list(ServiceBill.class, sql, paramMap);
    }

    public int count(List<Param> params, String serviceCode, String orderSn, Long userId, List<Long> projectIds)
        throws Exception {
        String subSql =
            "select  a.* from service_bill a left join service_order b on b.id = a.order_id left join tenant c on c.id = a.tenant_id where 1 = 1 ";
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        if (StringUtils.isNotEmpty(serviceCode)) {
            subSql += " and b.service_code = :serviceCode";
            paramMap.put("serviceCode", serviceCode);
        }
        if (StringUtils.isNotEmpty(orderSn)) {
            subSql += " and b.sn like '%" + orderSn + "%'";
        }
        subSql = QueryBuilder.buildRaw(subSql, params, null, "a");
        String sql = "select distinct count(1) from (" + subSql + ") d  where ( 1=-1 ";
        if (null != userId && 0L != userId) {
            sql += " or d.user_id =:userId ";
            paramMap.put("userId", userId);
        }
        if (null != projectIds && !projectIds.isEmpty()) {
            sql += " or d.project_id in (:projectIds) ";
            paramMap.put("projectIds", projectIds);
        }
        sql += " )";
        sql = QueryBuilder.buildRaw(sql, null, null, "d");
        return this.countQuery(sql, paramMap).intValue();
    }

    public int count(String status, String chargeMode, Long tenantId, Long userId, Long orderId) {
        String sql =
            "select count(a.id) from service_bill a left join service_order b on b.id = a.order_id where b.charge_mode = :chargeMode and a.status = :status and a.order_id = :orderId ";
        Map<String, Object> paramMap = MapTools.simpleMap("chargeMode", chargeMode);
        paramMap.put("status", status);
        paramMap.put("orderId", orderId);
        if (null != tenantId && 0L != tenantId) {
            sql += " and a.tenant_id = :tenantId";
            paramMap.put("tenantId", tenantId);
        }
        if (null != userId && 0L != userId) {
            sql += " and a.user_id = :userId";
            paramMap.put("userId", userId);
        }
        return this.countQuery(sql, paramMap).intValue();
    }

    public Boolean remove(Long id) throws Exception {
        String sql = "delete from service_bill where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        return this.execute(sql, params) > 0;
    }

    public List<ServiceBill> listByOrderId(Long orderId) throws Exception {
        String sql =
            "select a.*,c.name tenant_name from service_bill a left join tenant c on c.id = a.tenant_id where a.order_id = :orderId";
        Map<String, Object> params = MapTools.simpleMap("orderId", orderId);
        return this.list(ServiceBill.class, sql, params);
    }

    public List<ServiceBill> listByOrderId(Long orderId, String month) throws Exception {
        String sql =
            "select a.*,c.name tenant_name from service_bill a left join tenant c on c.id = a.tenant_id where a.order_id = :orderId and a.month = :month";
        Map<String, Object> params = MapTools.simpleMap("orderId", orderId);
        params.put("month", month);
        return this.list(ServiceBill.class, sql, params);
    }

    public Boolean removeByOrderId(Long orderId) throws Exception {
        String sql = "delete from service_bill where order_id = :orderId";
        Map<String, Object> params = MapTools.simpleMap("orderId", orderId);
        return this.execute(sql, params) > 0;
    }

    public Boolean removeByOrderIds(List<Long> orderIds) throws Exception {
        String sql = "delete from service_bill where order_id in (:orderIds)";
        Map<String, Object> params = MapTools.simpleMap("orderIds", orderIds);
        return this.execute(sql, params) > 0;
    }

    public ServiceBill query(Long id) throws Exception {
        String sql =
            "select a.*,b.charge_mode,b.sn order_sn,c.name tenant_name from service_bill a left join service_order b on b.id = a.order_id left join tenant c on c.id = a.tenant_id where a.id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<ServiceBill> list = this.list(ServiceBill.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    public ServiceBill queryBySn(String sn) throws Exception {
        String sql =
            "select a.*,b.charge_mode,b.sn order_sn,c.name tenant_name from service_bill a left join service_order b on b.id = a.order_id left join tenant c on c.id = a.tenant_id where a.sn = :sn";
        Map<String, Object> params = MapTools.simpleMap("sn", sn);
        List<ServiceBill> list = this.list(ServiceBill.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    public BigDecimal summary(List<Param> params) throws Exception {
        String sql = "select sum(money) summary from service_bill where status = 'paid'";
        sql = QueryBuilder.buildRaw(sql, params, null, "");
        Map<String, Object> param = QueryBuilder.getParam(params);
        List<Map<String, Object>> list = this.list(sql, param);
        Map<String, Object> resultMap = list.get(0);
        Object result = resultMap.get("summary");
        if (null == result) {
            return BigDecimal.valueOf(0);
        }
        return (BigDecimal)result;
    }

    public Double summary(List<Param> params, Long managerId, String city) throws Exception {
        String sql =
            "select sum(a.money) summary from service_bill a left join tenant b on b.id = a.tenant_id where a.status = 'paid'";
        Map<String, Object> param = QueryBuilder.getParam(params);
        if (null != managerId && 0L != managerId) {
            sql += " and b.manager_id = :managerId";
            param.put("managerId", managerId);
        }
        if (StringUtils.isNotEmpty(city)) {
            sql += " and b.city = :city";
            param.put("city", city);
        }
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        List<Map<String, Object>> list = this.list(sql, param);
        Map<String, Object> resultMap = list.get(0);
        Object result = resultMap.get("summary");
        if (null == result) {
            return 0.0;
        }
        BigDecimal decimal = (BigDecimal)result;
        return decimal.doubleValue();
    }

    public ServiceBill getMonthBill(Long orderId, String month) {
        String sql =
            "select a.*,c.name tenant_name from service_bill a left join tenant c on c.id = a.tenant_id where a.order_id = :orderId and a.month = :month limit 1";
        Map<String, Object> params = MapTools.simpleMap("orderId", orderId);
        params.put("month", month);
        List<ServiceBill> list = this.list(ServiceBill.class, sql, params);
        return ListTool.isEmpty(list) ? null : list.get(0);
    }

    public List<ServiceBill> trend(Date start, Date end, Long tenantId) throws Exception {
        String sql =
            "select a.*,c.name tenant_name from service_bill a left join tenant c on c.id = a.tenant_id where a.status = 'paid' ";
        Map<String, Object> params = new HashMap<>();
        if (null != start) {
            sql += " and a.gmt_create >= :start";
            params.put("start", start);
        }
        if (null != end) {
            sql += " and a.gmt_create <= :end";
            params.put("end", end);
        }
        if (null != tenantId && 0L != tenantId) {
            sql += " and a.tenant_id = :tenantId";
            params.put("tenantId", tenantId);
        }
        return this.list(ServiceBill.class, sql, params);
    }

    public List<Map<String, Object>> tenantTop5() throws Exception {
        String sql =
            "select sum(a.money) total,b.name tenant_name from service_bill a left join tenant b on b.id = a.tenant_id where a.status = 'paid' and  b.status = 'NORMAL' group by a.tenant_id order by total desc limit 5";
        return this.list(sql);
    }

    public List<Map<String, Object>> managerTop5(String start, String end, String city, Long managerId)
        throws Exception {
        StringBuilder builder = new StringBuilder(
            "select sum(a.money) total,b.manager_id,b.name manager_name from service_bill a left join tenant b on b.id = a.tenant_id left join manager c on c.id = b.manager_id where a.status = 'paid'  ");
        if (null != start) {
            builder.append(" and a.month >= '" + start + "'");
        }
        if (null != end) {
            builder.append(" and a.month <= '" + end + "'");
        }
        if (StringUtils.isNotEmpty(city)) {
            builder.append(" and b.city = '" + city + "'");
        }
        if (null != managerId && 0L != managerId) {
            builder.append(" and b.manager_id = " + managerId);
        }
        builder.append(" GROUP BY b.manager_id ORDER BY total desc LIMIT 5");
        return this.list(builder.toString());
    }

    public List<Map<String, Object>> cityTop5(String start, String end, String city, Long managerId) throws Exception {
        StringBuilder builder = new StringBuilder(
            "select sum(a.money) total,b.city from service_bill a left join tenant b on b.id = a.tenant_id where a.status = 'paid' ");
        if (null != start) {
            builder.append(" and a.month >= '" + start + "'");
        }
        if (null != end) {
            builder.append(" and a.month <= '" + end + "'");
        }
        if (StringUtils.isNotEmpty(city)) {
            builder.append(" and b.city = '" + city + "'");
        }
        if (null != managerId && 0L != managerId) {
            builder.append(" and b.manager_id = " + managerId);
        }
        builder.append(" GROUP BY b.city ORDER BY total desc LIMIT 5");
        return this.list(builder.toString());
    }

    public List<Map<String, Object>> tenantTop5(String start, String end, String city, Long tenantId) throws Exception {
        StringBuilder builder = new StringBuilder(
            "select sum(a.money) total,b.name tenant_name from service_bill a left join tenant b on b.id = a.tenant_id ");
        builder.append(" where 1=1");
        if (null != start) {
            builder.append(" and a.month >= '" + start + "'");
        }
        if (null != end) {
            builder.append(" and a.month <= '" + end + "'");
        }
        if (StringUtils.isNotEmpty(city)) {
            builder.append(" and b.city = '" + city + "'");
        }
        if (null != tenantId && 0L != tenantId) {
            builder.append(" and a.tenant_id = " + tenantId);
        }
        builder.append(" GROUP BY a.tenant_id ORDER BY total desc LIMIT 5");
        return this.list(builder.toString());
    }

    public List<ServiceBill> listByTenant(String start, String end, Long tenantId) throws Exception {
        StringBuilder builder = new StringBuilder(
            "select a.*,c.name tenant_name,u.name user_name,cv.name vendor_name,cv.id vendor_id, so.service_code service_code, cs.name service_name,b.name business_name,p.name project_name from service_bill a left join tenant c on c.id = a.tenant_id "
                + "left join service_order so on so.id = a.order_id left join cloud_service cs on cs.code = so.service_code "
                + "left join cloud_vendor cv on cv.id =so.vendor_id left join business b on b.id=a.business_id left join user u on a.user_id = u.id "
                + "left join project p on p.id=a.project_id where a.status = 'paid' ");
        Map<String, Object> params = new HashMap<>();
        if (null != tenantId && 0 != tenantId) {
            builder.append(" and a.tenant_id = :tenantId");
            params.put("tenantId", tenantId);
        }
        if (StringUtils.isNotEmpty(start)) {
            builder.append(" and a.gmt_create >= :start");
            params.put("start", start);
        }
        if (StringUtils.isNotEmpty(end)) {
            builder.append(" and a.gmt_create <= :end");
            params.put("end", end);
        }
        return this.list(ServiceBill.class, builder.toString(), params);
    }

    public List<ServiceBill> listByTenant(String start, String end, Long tenantId, Long vendorId) throws Exception {
        StringBuilder builder = new StringBuilder(
            "select a.*,c.name tenant_name,cv.name vendor_name,cv.id vendor_id, so.service_code service_code, cs.name service_name,b.name business_name,p.name project_name from service_bill a left join tenant c on c.id = a.tenant_id "
                + "left join service_order so on so.id = a.order_id left join cloud_service cs on cs.code = so.service_code left join cloud_vendor cv on cv.id =so.vendor_id left join business b on b.id=a.business_id left join project p on p.id=a.project_id where a.status = 'paid' ");
        Map<String, Object> params = new HashMap<>();
        if (null != tenantId && 0 != tenantId) {
            builder.append(" and a.tenant_id = :tenantId");
            params.put("tenantId", tenantId);
        }
        if (null != vendorId && 0 != vendorId) {
            builder.append(" and so.vendor_id = :vendorId");
            params.put("vendorId", vendorId);
        }
        if (StringUtils.isNotEmpty(start)) {
            builder.append(" and a.gmt_create >= :start");
            params.put("start", start);
        }
        if (StringUtils.isNotEmpty(end)) {
            builder.append(" and a.gmt_create <= :end");
            params.put("end", end);
        }
        return this.list(ServiceBill.class, builder.toString(), params);
    }

    public List<ServiceBill> listByOrder(String start, String end, Long orderId, Long vendorId) throws Exception {
        StringBuilder builder = new StringBuilder(
            "select a.*,c.name tenant_name,cv.name vendor_name,so.service_code service_code, cs.name service_name from service_bill a left join tenant c on c.id = a.tenant_id"
                + " left join service_order so on so.id = a.order_id left join cloud_service cs on cs.code = so.service_code left join cloud_vendor cv on cv.id =so.vendor_id where a.status = 'paid' ");
        Map<String, Object> params = new HashMap<>();
        if (null != orderId && 0 != orderId) {
            builder.append(" and a.order_id = :orderId");
            params.put("orderId", orderId);
        }
        if (null != vendorId && 0 != vendorId) {
            builder.append(" and so.vendor_id = :vendorId");
            params.put("vendorId", vendorId);
        }
        if (StringUtils.isNotEmpty(start)) {
            builder.append(" and a.gmt_create >= :start");
            params.put("start", start);
        }
        if (StringUtils.isNotEmpty(end)) {
            builder.append(" and a.gmt_create <= :end");
            params.put("end", end);
        }
        return this.list(ServiceBill.class, builder.toString(), params);
    }

    public List<ServiceBill> listByCode(String start, String end, String code, Long vendorId) throws Exception {
        StringBuilder builder = new StringBuilder(
            "select a.*,c.name tenant_name,cv.name vendor_name,so.service_code service_code, cs.name service_name,b.name business_name,p.name project_name from service_bill a "
                + "left join tenant c on c.id = a.tenant_id left join service_order so on so.id = a.order_id "
                + "left join cloud_service cs on cs.code = so.service_code left join project p on p.id=a.project_id "
                + "left join business b on b.id=a.business_id left join cloud_vendor cv on cv.id =so.vendor_id "
                + "where a.status = 'paid' ");
        Map<String, Object> params = new HashMap<>();
        if (StringUtils.isNotEmpty(code)) {
            builder.append(" and so.service_code = :code");
            params.put("code", code);
        }
        if (null != vendorId && 0 != vendorId) {
            builder.append(" and so.vendor_id = :vendorId");
            params.put("vendorId", vendorId);
        }
        if (StringUtils.isNotEmpty(start)) {
            builder.append(" and a.gmt_create >= :start");
            params.put("start", start);
        }
        if (StringUtils.isNotEmpty(end)) {
            builder.append(" and a.gmt_create <= :end");
            params.put("end", end);
        }
        return this.list(ServiceBill.class, builder.toString(), params);
    }

    public BigDecimal summary(Long tenantId) throws Exception {
        String sql = "select sum(money) summary from service_bill where status = 'paid' ";
        Map<String, Object> param = new HashMap<>();
        if (null != tenantId && 0L != tenantId) {
            sql += " and tenant_id = :tenantId";
            param.put("tenantId", tenantId);
        }
        List<Map<String, Object>> list = this.list(sql, param);
        Map<String, Object> resultMap = list.get(0);
        Object result = resultMap.get("summary");
        if (null == result) {
            return BigDecimal.valueOf(0);
        }
        return (BigDecimal)result;
    }

    public List<Map<String, Object>> projectTop5(List<Long> projectIds) throws Exception {
        String sql =
            "select sum(a.money) total,b.name project_name from service_bill a left join project b on b.id = a.project_id where a.project_id in (:projectIds) and a.status = 'paid'  group by a.project_id order by total desc limit 5";
        return this.list(sql, MapTools.simpleMap("projectIds", projectIds));
    }

    public List<Map<String, Object>> totalConsumption(String start, String end, String vendorType, Long tenantId) {
        StringBuilder builder = new StringBuilder(
            "select sum(a.money) total_money,cv.type vendor_type from bill_item a left join service_bill b on b.id = a.bill_id left join service_order so on so.id = b.order_id left join cloud_vendor cv on cv.id =so.vendor_id where b.status = 'paid' ");
        Map<String, Object> params = new HashMap<>();
        if (!org.springframework.util.StringUtils.isEmpty(vendorType)) {
            builder.append(" and cv.type = :vendorType");
            params.put("vendorType", vendorType);
        }
        if (!org.springframework.util.StringUtils.isEmpty(start)) {
            builder.append(" and b.gmt_create >= :start");
            params.put("start", start);
        }
        if (!org.springframework.util.StringUtils.isEmpty(end)) {
            builder.append(" and b.gmt_create <= :end");
            params.put("end", end);
        }
        if (null != tenantId && 0L != tenantId) {
            builder.append(" and b.tenant_id = :tenantId");
            params.put("tenantId", tenantId);
        }
        builder.append(" group by cv.type");
        return this.list(builder.toString(), params);
    }

    public List<Map<String, Object>> top5Consumption(String start, String end, String vendorType, Boolean groupByTenant,
        Long tenantId) {
        StringBuilder builder = new StringBuilder("select sum(a.money) total_money");
        if (groupByTenant) {
            builder.append(
                ",c.name tenant_name from bill_item a left join service_bill b on b.id = a.bill_id left join tenant c on c.id = b.tenant_id");
        } else {
            builder.append(
                ",c.name project_name from bill_item a left join service_bill b on b.id = a.bill_id left join project c on c.id = b.project_id");
        }
        builder.append(
            " left join service_order so on so.id = b.order_id left join cloud_vendor cv on cv.id = so.vendor_id where b.status = 'paid' ");
        Map<String, Object> params = new HashMap<>();
        if (!org.springframework.util.StringUtils.isEmpty(vendorType)) {
            builder.append(" and cv.type = :vendorType");
            params.put("vendorType", vendorType);
        }
        if (!org.springframework.util.StringUtils.isEmpty(start)) {
            builder.append(" and b.gmt_create >= :start");
            params.put("start", start);
        }
        if (!org.springframework.util.StringUtils.isEmpty(end)) {
            builder.append(" and b.gmt_create <= :end");
            params.put("end", end);
        }
        if (null != tenantId && 0L != tenantId) {
            builder.append(" and b.tenant_id = :tenantId");
            params.put("tenantId", tenantId);
        }
        if (groupByTenant) {
            builder.append(" group by b.tenant_id order by total_money desc limit 5");
        } else {
            builder.append(" group by b.project_id order by total_money desc limit 5");
        }
        return this.list(builder.toString(), params);
    }

    public List<Map<String, Object>> resourceConsumptionTrend(String start, String end, String vendorType,
        Long tenantId) {
        StringBuilder builder = new StringBuilder(
            "select sum(a.money) money,DATE_FORMAT(a.gmt_create,'%Y-%m-%d') gmt_create,cv.type vendor_type,so.service_code,b.name tenant_name from service_bill a left join tenant b on b.id = a.tenant_id left join service_order so on so.id = a.order_id left join cloud_vendor cv on cv.id = so.vendor_id where a.status = 'paid' ");
        Map<String, Object> params = new HashMap<>();
        if (!org.springframework.util.StringUtils.isEmpty(vendorType)) {
            builder.append(" and cv.type = :vendorType");
            params.put("vendorType", vendorType);
        }
        if (!org.springframework.util.StringUtils.isEmpty(start)) {
            builder.append(" and a.gmt_create >= :start");
            params.put("start", start);
        }
        if (!org.springframework.util.StringUtils.isEmpty(end)) {
            builder.append(" and a.gmt_create <= :end");
            params.put("end", end);
        }
        if (null != tenantId && 0L != tenantId) {
            builder.append(" and a.tenant_id = :tenantId");
            params.put("tenantId", tenantId);
        }
        builder.append(" group by DATE_FORMAT(a.gmt_create,'%Y-%m-%d'),cv.type");
        return this.list(builder.toString(), params);
    }

    public List<Map<String, Object>> projectConsumptionTrend(String start, String end, String vendorType,
        Long tenantId) {
        StringBuilder builder = new StringBuilder(
            "select sum(a.money) money,DATE_FORMAT(a.gmt_create,'%Y-%m-%d') gmt_create,cv.type vendor_type,so.service_code,b.name project_name from service_bill a left join project b on b.id = a.project_id left join service_order so on so.id = a.order_id left join cloud_vendor cv on cv.id = so.vendor_id where a.status = 'paid' and b.visibility = 'MAJOR_PROJECT' ");
        Map<String, Object> params = new HashMap<>();
        if (!org.springframework.util.StringUtils.isEmpty(vendorType)) {
            builder.append(" and cv.type = :vendorType");
            params.put("vendorType", vendorType);
        }
        if (!org.springframework.util.StringUtils.isEmpty(start)) {
            builder.append(" and a.gmt_create >= :start");
            params.put("start", start);
        }
        if (!org.springframework.util.StringUtils.isEmpty(end)) {
            builder.append(" and a.gmt_create <= :end");
            params.put("end", end);
        }
        if (null != tenantId && 0L != tenantId) {
            builder.append(" and a.tenant_id = :tenantId");
            params.put("tenantId", tenantId);
        }
        builder.append(" group by DATE_FORMAT(a.gmt_create,'%Y-%m-%d'),cv.type");
        return this.list(builder.toString(), params);
    }

    public List<ServiceBill> listByUser(String start, String end, Long userId, Long vendorId) throws Exception {
        StringBuilder builder = new StringBuilder(
            "select a.*,c.name user_name,cv.name vendor_name,cv.id vendor_id, so.service_code service_code, cs.name service_name,b.name business_name,p.name project_name from service_bill a left join user c on c.id = a.user_id "
                + "left join service_order so on so.id = a.order_id left join cloud_service cs on cs.code = so.service_code left join cloud_vendor cv on cv.id =so.vendor_id left join business b on b.id=a.business_id left join project p on p.id=a.project_id where a.status = 'paid' ");
        Map<String, Object> params = new HashMap<>();
        if (null != userId && 0 != userId) {
            builder.append(" and a.user_id = :userId");
            params.put("userId", userId);
        }
        if (null != vendorId && 0 != vendorId) {
            builder.append(" and so.vendor_id = :vendorId");
            params.put("vendorId", vendorId);
        }
        if (StringUtils.isNotEmpty(start)) {
            builder.append(" and a.gmt_create >= :start");
            params.put("start", start);
        }
        if (StringUtils.isNotEmpty(end)) {
            builder.append(" and a.gmt_create <= :end");
            params.put("end", end);
        }
        builder.append(" and a.user_id is not null");
        return this.list(ServiceBill.class, builder.toString(), params);
    }
}
