package com.bocloud.cos.repository;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.entity.ServiceOrder;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;
import org.springframework.util.StringUtils;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 订单数据访问
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年3月10日
 */
@Repository("serviceOrderRepository") public class ServiceOrderRepository extends BasicGenericDao<ServiceOrder, Long> {

    public ServiceOrderRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public List<ServiceOrder> list(int page, int rows, Map<String, String> sorter, List<Param> params, Long userId,
        List<Long> projectIds, String uuid) throws Exception {
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        String subquerySql;
        if (StringUtils.hasText(uuid)) {
            subquerySql =
                "select distinct a.*,b.name user_name,c.name tenant_name,d.name coupon_name,e.name service_name,p.name project_name from service_order a"
                    + " left join user b on b.id = a.user_id left join tenant c on c.id = a.tenant_id"
                    + " left join coupon d on d.id = a.coupon_id  left join cloud_service e on e.code = a.service_code "
                    + " left join project p on p.id = a.project_id left join order_item oi on a.id = oi.order_id where a.is_deleted = 0 and oi.sn like '%"+uuid+"%' ";
        } else {
            subquerySql =
                "select a.*,b.name user_name,c.name tenant_name,e.name service_name,p.name project_name from service_order a"
                    + " left join user b on b.id = a.user_id left join tenant c on c.id = a.tenant_id"
                    + " left join cloud_service e on e.code = a.service_code "
                    + " left join project p on p.id = a.project_id where a.is_deleted = 0 ";
        }
        subquerySql = QueryBuilder.buildRaw(subquerySql, params, null, "a");
        String sql = "select distinct d.* from (" + subquerySql + ") d  where ( 1=-1 ";
        if (null != userId && 0L != userId) {
            sql += " or d.user_id =:userId ";
            paramMap.put("userId", userId);
        }
        if (null != projectIds && !projectIds.isEmpty()) {
            sql += " or d.project_id in (:projectIds) ";
            paramMap.put("projectIds", projectIds);
        }
        sql += " )";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, null, sorter), "d");
        return this.list(ServiceOrder.class, sql, paramMap);
    }

    public int count(List<Param> params, Long userId, List<Long> projectIds, String uuid) {
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        String subquerySql;
        if (StringUtils.hasText(uuid)) {
            subquerySql =
                "select distinct a.* from service_order a left join order_item b on a.id = b.order_id where a.is_deleted = 0 and b.sn like '%"+uuid+"%' ";
        } else {
            subquerySql = "select a.* from service_order a where a.is_deleted = 0  ";
        }
        subquerySql = QueryBuilder.buildRaw(subquerySql, params, null, "a");
        String sql = "select distinct count(1) from (" + subquerySql + ") c  where ( 1=-1 ";
        if (null != userId && 0L != userId) {
            sql += " or c.user_id =:userId";
            paramMap.put("userId", userId);
        }
        if (null != projectIds && !projectIds.isEmpty()) {
            sql += " or c.project_id in (:projectIds) ";
            paramMap.put("projectIds", projectIds);
        }
        sql += ")";
        return this.countQuery(sql, paramMap).intValue();
    }

    public List<ServiceOrder> list(int page, int rows, Map<String, String> sorter, List<Param> params)
        throws Exception {
        String sql =
            "select a.*,b.name user_name,c.name tenant_name,d.name coupon_name,e.name service_name,p.name project_name from service_order a"
                + " left join user b on b.id = a.user_id left join tenant c on c.id = a.tenant_id"
                + " left join coupon d on d.id = a.coupon_id" + " left join cloud_service e on e.code = a.service_code "
                + " left join project p on p.id = a.project_id where a.is_deleted = 0 ";
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        return this.list(ServiceOrder.class, sql, paramMap);
    }

    public List<ServiceOrder> list(int page, int rows, Map<String, String> sorter, List<Param> params, String uuid)
        throws Exception {
        String sql;
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        if (StringUtils.hasText(uuid)) {
            sql =
                "select distinct a.*,b.name user_name,c.name tenant_name,d.name coupon_name,e.name service_name,p.name project_name from service_order a"
                    + " left join user b on b.id = a.user_id left join tenant c on c.id = a.tenant_id"
                    + " left join coupon d on d.id = a.coupon_id  left join cloud_service e on e.code = a.service_code "
                    + " left join project p on p.id = a.project_id left join order_item oi on a.id = oi.order_id where a.is_deleted = 0 and oi.sn like '%"+uuid+"%' ";
        } else {
            sql =
                "select a.*,b.name user_name,c.name tenant_name,d.name coupon_name,e.name service_name,p.name project_name from service_order a"
                    + " left join user b on b.id = a.user_id left join tenant c on c.id = a.tenant_id"
                    + " left join coupon d on d.id = a.coupon_id"
                    + " left join cloud_service e on e.code = a.service_code "
                    + " left join project p on p.id = a.project_id where a.is_deleted = 0 ";
        }

        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        return this.list(ServiceOrder.class, sql, paramMap);
    }

    public int count(List<Param> params) {
        String sql = "select count(id) from service_order a where a.is_deleted = 0  ";
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        return this.countQuery(sql, paramMap).intValue();
    }

    public int count(List<Param> params, String uuid) {
        String sql;
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        if (StringUtils.hasText(uuid)) {
            sql =
                "select distinct count(a.id) from service_order a left join order_item b on a.id = b.order_id where a.is_deleted = 0 and b.sn like '%"+uuid+"%'  ";
        } else {
            sql = "select count(a.id) from service_order a where a.is_deleted = 0";
        }
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        return this.countQuery(sql, paramMap).intValue();
    }

    public Boolean remove(Long id) {
        String sql = "update service_order set is_deleted = 1, gmt_modify = :gmtModify where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("gmtModify", new Date());
        return this.execute(sql, params) > 0;
    }

    public Boolean batchRemove(List<Long> ids) {
        String sql = "update service_order set is_deleted = 1, gmt_modify = :gmtModify where id in (:ids)";
        Map<String, Object> params = MapTools.simpleMap("ids", ids);
        params.put("gmtModify", new Date());
        return this.execute(sql, params) > 0;
    }

    /**
     * 查看租户下的服务订单(包括租户下用户的订单)
     *
     * @param tenantId
     * @return
     * @throws Exception
     */
    public List<ServiceOrder> listByTenantId(Long tenantId) {
        String sql =
            "select a.*,b.name user_name,c.name tenant_name,d.name coupon_name from service_order a left join user b on b.id = a.user_id left join tenant c on c.id = a.tenant_id left join coupon d on d.id = a.coupon_id where a.is_deleted = 0  AND a.tenant_id = :tenantId";
        Map<String, Object> params = MapTools.simpleMap("tenantId", tenantId);
        return this.list(ServiceOrder.class, sql, params);
    }

    /**
     * 查看租户自身的服务订单（不包括租户下用户的订单）
     *
     * @param tenantId    租户id
     * @param serviceCode 服务编码
     * @return 订单列表
     * @throws Exception
     */
    public List<ServiceOrder> listTenantServiceOrder(Long tenantId, String serviceCode) {
        String sql =
            "select a.*,b.name user_name,c.name tenant_name,d.name coupon_name from service_order a left join user b on b.id = a.user_id left join tenant c on c.id = a.tenant_id left join coupon d on d.id = a.coupon_id where a.tenant_id = :tenantId and a.user_id is null and a.service_code = :serviceCode";
        Map<String, Object> params = MapTools.simpleMap("tenantId", tenantId);
        params.put("serviceCode", serviceCode);
        return this.list(ServiceOrder.class, sql, params);
    }

    public List<ServiceOrder> listByUserId(Long userId) {
        String sql =
            "select a.*,b.name user_name,c.name tenant_name,d.name coupon_name from service_order a left join user b on b.id = a.user_id left join tenant c on c.id = a.tenant_id left join coupon d on d.id = a.coupon_id where a.is_deleted = 0  AND a.user_id = :userId";
        Map<String, Object> params = MapTools.simpleMap("userId", userId);
        return this.list(ServiceOrder.class, sql, params);
    }

    public ServiceOrder query(Long id) {
        String sql =
            "select a.*,b.name user_name,c.name tenant_name,d.name coupon_name,p.name project_name from service_order a left join user b on b.id = a.user_id left join tenant c on c.id = a.tenant_id left join coupon d on d.id = a.coupon_id left join project p on p.id = a.project_id where a.id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<ServiceOrder> list = this.list(ServiceOrder.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    public ServiceOrder queryBySn(String sn) {
        String sql =
            "select a.*,b.name user_name,c.name tenant_name,d.name coupon_name,s.name service_name from service_order a"
                + " left join user b on b.id = a.user_id left join tenant c on c.id = a.tenant_id"
                + " left join coupon d on d.id = a.coupon_id left join cloud_service s on s.code = a.service_code where a.sn = :sn";
        Map<String, Object> params = MapTools.simpleMap("sn", sn);
        List<ServiceOrder> list = this.list(ServiceOrder.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    public List<ServiceOrder> orders(String chargeMode) {
        String sql =
            "select DISTINCT a.*,b.name user_name,c.name tenant_name,d.name coupon_name from service_order a left join user b on b.id = a.user_id left join tenant c on c.id = a.tenant_id left join coupon d on d.id = a.coupon_id where a.charge_mode = :chargeMode";
        Map<String, Object> params = MapTools.simpleMap("chargeMode", chargeMode);
        return this.list(ServiceOrder.class, sql, params);
    }

    public List<Map<String, Object>> orderPaidStatistics(String start, String end, Long tenantId) {
        String sql =
            "select count(a.id) num,a.order_status,c.name tenant_name from service_order a left join tenant c on c.id = a.tenant_id where a.is_deleted = 0 ";
        Map<String, Object> params = new HashMap<>(4);
        if (StringUtils.hasText(start)) {
            sql += " and a.gmt_create >= :start";
            params.put("start", start);
        }
        if (StringUtils.hasText(end)) {
            sql += " and a.gmt_create <= :end";
            params.put("end", end);
        }
        if (null != tenantId && 0L != tenantId) {
            sql += " and a.tenant_id = :tenantId";
            params.put("tenantId", tenantId);
        }
        sql += " group by a.order_status";
        return this.list(sql, params);
    }

    public List<ServiceOrder> timeBetweenOrders(String start, String end, Long tenantId, Long userId) {
        String sql =
            "select a.*,c.name service_name,u.name user_name , t.name tenant_name from service_order a left join cloud_service c on a.service_code = c.code "
                + " left join user u on u.id = a.user_id  left join tenant t on t.id = a.tenant_id  where a.is_deleted = 0 ";
        Map<String, Object> params = new HashMap<>();
        if (StringUtils.hasText(start)) {
            sql += " and a.gmt_create >= :start";
            params.put("start", start);
        }
        if (StringUtils.hasText(end)) {
            sql += " and a.gmt_create <= :end";
            params.put("end", end);
        }
        if (null != tenantId && 0L != tenantId) {
            sql += " and a.tenant_id = :tenantId";
            params.put("tenantId", tenantId);
        }
        if (null != userId && 0L != userId) {
            sql += " and a.user_id = :userId";
            params.put("userId", userId);
        }
        return this.list(ServiceOrder.class, sql, params);
    }

    public List<Map<String, Object>> top5(String start, String end, Boolean tenant, Boolean zone) {
        StringBuilder builder = new StringBuilder("select count(a.id) total,b.");
        if (null != tenant && tenant) {
            builder.append("name tenant_name");
        } else if (null != zone && zone) {
            builder.append("city");
        }
        builder.append(" from service_order a join tenant b on b.id = a.tenant_id where 1 = 1");
        Map<String, Object> paramMap = new HashMap<>();
        if (null != start) {
            builder.append(" and a.gmt_create >= :start");
            paramMap.put("start", start);
        }
        if (null != end) {
            builder.append(" and a.gmt_create <= :end");
            paramMap.put("end", end);
        }
        if (null != zone && zone) {
            builder.append(" and b.city is not null");
        }
        if (null != zone && zone) {
            builder.append(" group by b.city order by total desc limit 5");
        } else {
            builder.append(" group by a.tenant_id order by total desc limit 5");
        }
        return this.list(builder.toString(), paramMap);
    }

    public int countByContext(String status, Long tenantId, Long userId) {
        String sql = "select count(id) from service_order a where a.is_deleted = 0 ";
        Map<String, Object> paramMap = MapTools.simpleMap("tenantId", tenantId);
        if (null != userId && 0L != userId) {
            sql += " and a.user_id = :userId";
            paramMap.put("userId", userId);
        }
        if (!StringUtils.isEmpty(status)) {
            sql += " and a.order_status = :status";
            paramMap.put("status", status);
        }
        return this.countQuery(sql, paramMap).intValue();
    }

    public List<ServiceOrder> listByVendor(String start, String end, Long vendorId) throws Exception {
        String sql =
            "select a.*,cv.name vendor_name,cs.name service_name from service_order a left join cloud_vendor cv on a.vendor_id = cv.id "
                + "left join cloud_service cs on cs.code=a.service_code where a.is_deleted = 0  ";
        Map<String, Object> params = new HashMap<>();
        if (null != vendorId && 0 != vendorId) {
            sql += " and a.vendor_id = :vendorId";
            params.put("vendorId", vendorId);
        }
        if (!StringUtils.isEmpty(start)) {
            sql += " and a.gmt_create >= :start";
            params.put("start", start);
        }
        if (!StringUtils.isEmpty(end)) {
            sql += " and a.gmt_create <= :end";
            params.put("end", end);
        }
        return this.list(ServiceOrder.class, sql, params);
    }

    public List<ServiceOrder> listByTenantId(String start, String end, Long tenantId) throws Exception {
        String sql =
            "select a.*,cv.name vendor_name,cs.name service_name from service_order a left join cloud_vendor cv on a.vendor_id = cv.id "
                + "left join cloud_service cs on cs.code=a.service_code left join tenant t on a.tenant_id = t.id where a.is_deleted = 0 ";
        Map<String, Object> params = new HashMap<>();
        if (null != tenantId && 0 != tenantId) {
            sql += " and a.tenant_id = :tenantId";
            params.put("tenantId", tenantId);
        }
        if (!StringUtils.isEmpty(start)) {
            sql += " and a.gmt_create >= :start";
            params.put("start", start);
        }
        if (!StringUtils.isEmpty(end)) {
            sql += " and a.gmt_create <= :end";
            params.put("end", end);
        }
        return this.list(ServiceOrder.class, sql, params);
    }

    public List<ServiceOrder> listByTenantIds(String start, String end, List<Long> tenantIds) throws Exception {
        String sql =
            "select a.*,cv.name vendor_name,cs.name service_name from service_order a left join cloud_vendor cv on a.vendor_id = cv.id "
                + "left join cloud_service cs on cs.code=a.service_code left join tenant t on a.tenant_id = t.id where a.is_deleted = 0  ";
        Map<String, Object> params = new HashMap<>();
        if (!tenantIds.isEmpty()) {
            sql += " and a.tenant_id in (:tenantIds)";
            params.put("tenantIds", tenantIds);
        }
        if (!StringUtils.isEmpty(start)) {
            sql += " and a.gmt_create >= :start";
            params.put("start", start);
        }
        if (!StringUtils.isEmpty(end)) {
            sql += " and a.gmt_create <= :end";
            params.put("end", end);
        }
        return this.list(ServiceOrder.class, sql, params);
    }

    public List<ServiceOrder> listByCode(String start, String end, String serviceCode) throws Exception {
        String sql =
            "select a.*,cv.name vendor_name,cs.name service_name from service_order a left join cloud_vendor cv on a.vendor_id = cv.id "
                + "left join cloud_service cs on cs.code=a.service_code where a.is_deleted = 0  ";
        Map<String, Object> params = new HashMap<>();
        if (!StringUtils.isEmpty(serviceCode)) {
            sql += " and a.service_code = :serviceCode";
            params.put("serviceCode", serviceCode);
        }
        if (!StringUtils.isEmpty(start)) {
            sql += " and a.gmt_create >= :start";
            params.put("start", start);
        }
        if (!StringUtils.isEmpty(end)) {
            sql += " and a.gmt_create <= :end";
            params.put("end", end);
        }
        return this.list(ServiceOrder.class, sql, params);
    }

    public List<ServiceOrder> listByStatus(String start, String end, String status) throws Exception {
        String sql =
            "select a.*,cv.name vendor_name,cs.name service_name from service_order a left join cloud_vendor cv on a.vendor_id = cv.id "
                + "left join cloud_service cs on cs.code=a.service_code where a.is_deleted = 0  ";
        Map<String, Object> params = new HashMap<>();
        if (!StringUtils.isEmpty(status)) {
            sql += " and a.order_status = :status";
            params.put("status", status);
        }
        if (!StringUtils.isEmpty(start)) {
            sql += " and a.gmt_create >= :start";
            params.put("start", start);
        }
        if (!StringUtils.isEmpty(end)) {
            sql += " and a.gmt_create <= :end";
            params.put("end", end);
        }
        return this.list(ServiceOrder.class, sql, params);
    }

    public List<ServiceOrder> listByType(String start, String end, String type) {
        String sql =
            "select a.*,cv.name vendor_name,cs.name service_name from service_order a left join cloud_vendor cv on a.vendor_id = cv.id "
                + "left join cloud_service cs on cs.code=a.service_code where a.is_deleted = 0  ";
        Map<String, Object> params = new HashMap<>();
        if (!StringUtils.isEmpty(type)) {
            sql += " and a.charge_mode = :type";
            params.put("type", type);
        }
        if (!StringUtils.isEmpty(start)) {
            sql += " and a.gmt_create >= :start";
            params.put("start", start);
        }
        if (!StringUtils.isEmpty(end)) {
            sql += " and a.gmt_create <= :end";
            params.put("end", end);
        }
        return this.list(ServiceOrder.class, sql, params);
    }
}
