package com.bocloud.cos.repository;

import com.bocloud.cos.common.utils.ArrayFormatter;
import com.bocloud.cos.enums.ReportFrequenceType;
import com.bocloud.cos.model.TenantInformationReport;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * @author lyy
 */
@Repository("tenantformInfoReportRepository")
public class TenantInfoReportRepository extends BasicGenericDao<TenantInformationReport, Long> {
    public TenantInfoReportRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    /**
     * since 2014-10-19 modify by sxp 兴业银行poc需求
     */
    public List<TenantInformationReport> queryTenantInformationReport(String[] resourceIds,
                                                                      ReportFrequenceType reportFrequenceType, Map<String, Object> param) throws Exception {
        param.put("ids", ArrayFormatter.toLongList(resourceIds));
        String timeClause = "";
        String timeCondition = "";

        // 加时间条件不合适，注释掉
        /*
         * switch (reportFrequenceType) { case ByDay: timeClause =
         * " and gmt_create >=:startTime and gmt_create <= :endTime "; timeCondition =
         * " and b.gmt_create >=:startTime and b.gmt_create <= :endTime "; break; case
         * ByWeek: timeClause =
         * " and DATE_FORMAT(gmt_create,'%Y-%m-%d') in (:dayList) "; timeCondition =
         * " and DATE_FORMAT(b.gmt_create,'%Y-%m-%d') in (:dayList) "; break; case
         * ByMonth: timeClause =
         * " and gmt_create >=:startDate and gmt_create< :endDate "; timeCondition =
         * " and b.gmt_create >=:startDate and b.gmt_create< :endDate "; break; default:
         * break; }
         */
        String sql = "SELECT t.id, t.`name`, DATEDIFF(CURDATE(), t.gmt_create) AS applyLength, "
                + "	t.gmt_create, IFNULL(s.cloudServerCount, 0) AS cloudServerCount, IFNULL(vo.volumeCount, 0) AS volumeCount, "
                + "	IFNULL(p.productCount, 0) AS productCount, IFNULL(a.parentIdNumber,0) AS parentIdNumber, IFNULL(b.userNumber,0) AS userNumber, IFNULL(c.projectNumber,0) AS projectNumber FROM tenant t "
                + " LEFT JOIN (SELECT parent_id,count(*) AS parentIdNumber FROM tenant WHERE is_deleted = 0 GROUP BY parent_id) a ON t.id = a.parent_id "
                + " LEFT JOIN(SELECT tenant_id, COUNT(*) AS userNumber FROM `user` WHERE is_deleted = 0  GROUP BY tenant_id)b ON t.id = b.tenant_id "
                + " LEFT JOIN (SELECT tenant_id,COUNT(*) AS projectNumber FROM project WHERE is_deleted = 0 GROUP BY tenant_id)c ON t.id = c.tenant_id "
                + " LEFT JOIN (SELECT cs.tenant_id, COUNT(cs.id) cloudServerCount "
                + " FROM cloud_server cs WHERE is_deleted = 0 " + timeClause + // 时间条件
                " GROUP BY cs.tenant_id ) s ON t.id = s.tenant_id "
                + " LEFT JOIN (SELECT v.tenant_id, COUNT(v.id) volumeCount FROM volume v WHERE v.is_deleted = 0 "
                + timeClause + // 时间条件
                " GROUP BY v.tenant_id ) vo ON t.id = vo.tenant_id LEFT JOIN (SELECT id, sum(count) productCount FROM "
                + " ( (SELECT a.id,COUNT(b.id) count FROM tenant a INNER JOIN cloud_server b ON a.id = b.tenant_id WHERE "
                + " a.is_deleted = 0 " + timeCondition + // 时间条件
                " GROUP BY b.tenant_id) UNION ALL (SELECT a.id, COUNT(b.id) count FROM tenant a "
                + " INNER JOIN volume b ON a.id = b.tenant_id WHERE b.is_deleted = 0 " + timeCondition + // 时间条件
                " GROUP BY b.tenant_id) UNION ALL (SELECT a.id, "
                + "	COUNT(b.id) count FROM tenant a INNER JOIN snapshot b ON a.id = b.tenant_id WHERE b.is_deleted = 0 "
                + timeCondition + " GROUP BY "
                + "	b.tenant_id) UNION ALL (SELECT a.id, COUNT(b.id) count FROM tenant a INNER JOIN router b ON a.id = b.tenant_id WHERE "
                + " b.is_deleted = 0 " + timeCondition + // 时间条件
                " GROUP BY b.tenant_id) UNION ALL (SELECT a.id,COUNT(b.id) count FROM tenant a "
                + " INNER JOIN floating_ip b ON a.id = b.tenant_id WHERE b.is_deleted = 0  " + timeCondition + // 时间条件
                " GROUP BY b.tenant_id)) x	GROUP BY x.id) p ON t.id = p.id where t.id in (:ids)";
        return this.list(TenantInformationReport.class, sql, param);
    }
}
