package com.bocloud.cos.intf;

import java.util.List;

import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.Result;
import com.bocloud.cos.model.AlterationParam;
import com.bocloud.cos.model.ApplicationParam;
import com.bocloud.cos.model.CancelationParam;
import com.bocloud.cos.model.ExtensionParam;
import com.bocloud.cos.model.OrderMessage;
import com.bocloud.cos.params.CloneParams;
import com.bocloud.cos.service.utils.AlterationModel;
import com.bocloud.cos.service.utils.ApplicationModel;

/**
 * 服务操作接口
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年4月15日
 */
public interface ResourceApplyService {

	/**
	 * 服务申请
	 *
	 * @param params
	 * @param context
	 * @return
	 */
	Result application(ApplicationParam params, RequestContext context);

	/**
	 * 服务申请校验
	 *
	 * @param param
	 * @param context
	 * @param isCheck
	 * @return
	 */
	BocloudResult<ApplicationModel> applyCheck(ApplicationParam param, RequestContext context, boolean isCheck);

	/**
	 * 服务变更
	 *
	 * @param params
	 * @param context
	 * @return
	 */
	Result alteration(AlterationParam params, RequestContext context);

	/**
	 * 服务变更校验
	 *
	 * @param param
	 * @param context
	 * @return
	 */
	BocloudResult<AlterationModel> alterCheck(AlterationParam param, RequestContext context);

	/**
	 * 服务延期
	 *
	 * @param params
	 * @param context
	 * @return
	 */
	Result extension(ExtensionParam params, RequestContext context);

	/**
	 * 服务退订
	 *
	 * @param params
	 * @param context
	 * @return
	 */
	Result cancelation(CancelationParam params, RequestContext context);

	/**
	 * 更新订单状态(k8s调用)
	 *
	 * @param message
	 * @return
	 */
	Result updateOrder(OrderMessage message);

	/**
	 * k8s缩容
	 *
	 * @param params
	 * @return
	 */
	Result k8sAlteration(AlterationParam params);

	/**
	 * 克隆虚拟机
	 *
	 * @param configs
	 * @param context
	 * @return
	 */
	Result clone(CloneParams configs, RequestContext context);

	/**
	 * 清空回收站
	 *
	 * @param context
	 * @return
	 */
	Result empty(RequestContext context);

}
