package com.bocloud.cos.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.cos.entity.Paragraph;
import com.bocloud.cos.entity.ReportConfigTemplate;
import com.bocloud.cos.entity.ReportHistoryResult;
import com.bocloud.cos.entity.ReportTemplateHistory;
import com.bocloud.cos.enums.ReportFrequenceType;
import com.bocloud.cos.intf.ChargeReportService;
import com.bocloud.cos.model.ChargeInformationReport;
import com.bocloud.cos.model.ReportPair;
import com.bocloud.cos.repository.ChargeReportRepository;
import com.bocloud.cos.repository.ParagraphRepository;
import com.bocloud.cos.repository.ReportConfigTemplateRepository;
import com.bocloud.cos.repository.ReportHistoryResultRepository;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.InputStream;
import java.util.*;

/**
 * @Author lyy
 */
@Service("chargeReportService")
public class ChargeReportServiceImpl extends AbstractReportService implements ChargeReportService {
	private static final Logger logger = LoggerFactory.getLogger(TenantReportServiceImpl.class);
	@Autowired
	private ReportConfigTemplateRepository reportConfigTemplateRepository1;
	@Autowired
	private ParagraphRepository paragraphDao;
	@Autowired
	private ChargeReportRepository chargeReportDao;

	@Autowired
	private ReportHistoryResultRepository reportHistoryResultDao;

	@Override
	public BocloudResult<List<ReportPair<List<ChargeInformationReport>>>> queryDetail(Long configTemplateId,
			ReportTemplateHistory history, Boolean isRealTime, RequestContext context) {
		List<ReportPair<List<ChargeInformationReport>>> resultList = new ArrayList<>();
		ReportConfigTemplate configTemplate = null;
		ReportFrequenceType reportFrequenceType = null;
		Map<String, Object> param = new HashMap<>(16);
		try {
			if (isRealTime != null && isRealTime) {
				// 查看实时数据
				configTemplate = reportConfigTemplateRepository1.query(configTemplateId);
				String cycleType = configTemplate.getCycleType();
				reportFrequenceType = ReportFrequenceType.valueOf(cycleType);
				// 处理时间参数
				extractTimeParam(null, reportFrequenceType, configTemplate, param, true);
			} else if (history != null) {
				// 查看历史数据
				configTemplateId = history.getConfigTemplateId();
				configTemplate = reportConfigTemplateRepository1.query(configTemplateId);
				String cycleType = configTemplate.getCycleType();
				reportFrequenceType = ReportFrequenceType.valueOf(cycleType);
				// 处理时间参数
				extractTimeParam(history.getTriggerTime(), reportFrequenceType, configTemplate, param, false);
			}
			try {
				List<Paragraph> paragraphs = paragraphDao.getByConfigTemplate(configTemplateId);
				for (Paragraph paragraph : paragraphs) {
					String resourceIdStr = paragraph.getResourceIds();
					String[] resourceIds = resourceIdStr.split(",");
					// 先查询出表格数据
					// List<ChargeInformationReport> reports =
					// chargeReportDao.queryChargeReport(resourceIds,
					// reportFrequenceType, param, context);
					List<ChargeInformationReport> reports = getMainTableReport(reportFrequenceType, param, resourceIds,
							context, null, configTemplate, paragraph, history);
					resultList.add(new ReportPair<>(paragraph.getName(), reports));
				}
			} catch (Exception e) {
				logger.error(ExceptionUtils.getFullStackTrace(e));
				return new BocloudResult<>(false, "查询失败:"+e.getMessage());
			}
		} catch (Exception e) {
			logger.error(ExceptionUtils.getFullStackTrace(e));
			return new BocloudResult<>(false, "查询失败:"+e.getMessage());
		}
		return new BocloudResult<>(true, resultList, "查询成功");
	}

	@Override
	public InputStream getMainTableJasperStream() throws Exception {
		return this.getClass().getClassLoader().getResourceAsStream("template/charge/main.jasper");
	}

	@Override
	public String getTitleName() {
		return "计量计费报表";
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<ChargeInformationReport> getMainTableReport(ReportFrequenceType reportFrequenceType,
			Map<String, Object> param, String[] resourceIds, RequestContext context, Boolean isAdmin,
			ReportConfigTemplate configTemplate, Paragraph paragraph, ReportTemplateHistory history) {
		@SuppressWarnings("rawtypes")
		List list = Collections.emptyList();
		try {
			/**
			 * 查询缓存的结果
			 */
			ReportHistoryResult cacheResult = reportHistoryResultDao.query(history.getId(), paragraph.getId(), 0L);
			if (cacheResult == null) {
				/**
				 * 缓存为空,计算数据，存入缓存
				 */
				list = chargeReportDao.queryChargeReport(resourceIds, reportFrequenceType, param, context);
				cacheResult = new ReportHistoryResult();
				String content = JSONObject.toJSONString(list);
				cacheResult.setContent(content);
				cacheResult.setHistoryId(history.getId());
				cacheResult.setContentTemplateId(0L);
				cacheResult.setParagraphId(paragraph.getId());
				reportHistoryResultDao.save(cacheResult);
			} else {
				list = JSONArray.parseArray(cacheResult.getContent(), ChargeInformationReport.class);
			}

		} catch (Exception e) {
			logger.error(ExceptionUtils.getFullStackTrace(e));
		}
		return list;
	}
}
