package com.bocloud.cos.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.ListTool;
import com.bocloud.cos.entity.*;
import com.bocloud.cos.enums.ChartType;
import com.bocloud.cos.enums.ReportFrequenceType;
import com.bocloud.cos.intf.PlatformInfoReportService;
import com.bocloud.cos.model.GraphData;
import com.bocloud.cos.model.PlatformInfoReport;
import com.bocloud.cos.model.ReportPair;
import com.bocloud.cos.repository.PlatformInfoReportRepository;
import com.bocloud.cos.repository.ReportHistoryResultRepository;
import com.google.common.collect.Lists;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.InputStream;
import java.util.*;

/**
 * @author hufy
 */
@Service("platformInfoReportService")
public class PlatformInfoReportServiceImpl extends AbstractReportService implements PlatformInfoReportService {

	private final Logger logger = LoggerFactory.getLogger(this.getClass());

	@Autowired
	PlatformInfoReportRepository platformInfoReportRepository;

	@Autowired
	ReportHistoryResultRepository reportHistoryResultRepository;

	@Override
	public BocloudResult<List<ReportPair<List<PlatformInfoReport>>>> queryDetail(Long configTemplateId,
			ReportTemplateHistory history, Boolean isRealTime) {
		List<ReportPair<List<PlatformInfoReport>>> resultList = new ArrayList<>();
		try {
			ReportConfigTemplate configTemplate = null;
			ReportFrequenceType reportFrequenceType = null;
			Map<String, Object> param = new HashMap<>(16);
			if (isRealTime != null && isRealTime) {
				// 查看实时数据
				configTemplate = reportConfigTemplateRepository.query(configTemplateId);
				String cycleType = configTemplate.getCycleType();
				reportFrequenceType = ReportFrequenceType.valueOf(cycleType);
				// 处理时间参数
				extractTimeParam(null, reportFrequenceType, configTemplate, param, true);
			} else if (history != null) {
				// 查看历史数据
				configTemplateId = history.getConfigTemplateId();
				configTemplate = reportConfigTemplateRepository.query(configTemplateId);
				String cycleType = configTemplate.getCycleType();
				reportFrequenceType = ReportFrequenceType.valueOf(cycleType);
				// 处理时间参数
				extractTimeParam(history.getTriggerTime(), reportFrequenceType, configTemplate, param, false);
			}
			List<Paragraph> paragraphs = paragraphRepository.getByConfigTemplate(configTemplateId);
			for (Paragraph paragraph : paragraphs) {
				String resourceIdStr = paragraph.getResourceIds();
				String[] resourceIds = resourceIdStr.split(",");
				// 先查询出表格数据
				List<PlatformInfoReport> reports = getMainTableReport(reportFrequenceType, param, resourceIds, null,
						null, configTemplate, paragraph, history);
				ReportPair<List<PlatformInfoReport>> report = new ReportPair<>(paragraph.getName(), reports);
				resultList.add(report);
				// 查询选择的图表模板数据
				List<ContentTemplate> contentTemplates = contentTemplateRepository
						.queryContentByParagraphId(paragraph.getId());
				if (!ListTool.isEmpty(contentTemplates)) {
					List<ReportPair.ChartContent> chartDatas = Lists.newArrayList();
					for (ContentTemplate contentTemplate : contentTemplates) {
						List<GraphData> graphDatas;
						try {
							assert history != null;
							ReportHistoryResult cacheResult = reportHistoryResultRepository.query(history.getId(),
									paragraph.getId(), contentTemplate.getId());
							if (cacheResult == null) {
								if ("虚拟机创建趋势图".equals(contentTemplate.getName())) {
									graphDatas = this.getVmLineChartByTime(reportFrequenceType, param, resourceIds);
								} else {
									return null;
								}
								if (graphDatas == null) {
									graphDatas = Collections.emptyList();
								}
								cacheResult = new ReportHistoryResult();
								String content = JSONObject.toJSONString(graphDatas);
								cacheResult.setContent(content);
								cacheResult.setHistoryId(history.getId());
								cacheResult.setContentTemplateId(contentTemplate.getId());
								cacheResult.setParagraphId(paragraph.getId());
								// 如果不存在则保存数据
								reportHistoryResultRepository.save(cacheResult);
							} else {
								graphDatas = JSONArray.parseArray(cacheResult.getContent(), GraphData.class);
							}
							ReportPair.ChartContent chartContent = new ReportPair.ChartContent();
							chartContent.setChartType(ChartType.valueOf(contentTemplate.getChartType()));
							chartContent.setGraphDatas(graphDatas);
							chartContent.setName(contentTemplate.getName());
							chartDatas.add(chartContent);
						} catch (Exception e) {
							logger.error(ExceptionUtils.getFullStackTrace(e));
						}
					}
					report.setContent(chartDatas);
				}
			}
		} catch (Exception e) {
			logger.error(ExceptionUtils.getFullStackTrace(e));
			return new BocloudResult<>(false, "查询失败:"+e.getMessage());
		}
		return new BocloudResult<>(true, resultList, "");
	}

	@Override
	public List<GraphData> getVmLineChartByTime(ReportFrequenceType frequenceType, Map<String, Object> params,
			String[] resourceIds) {
		List<GraphData> list = null;
		try {
			list = platformInfoReportRepository.queryVmLineChartByTime(frequenceType, params, resourceIds);
		} catch (Exception e) {
			logger.error(ExceptionUtils.getFullStackTrace(e));
		}
		return list;
	}

	@Override
	public InputStream getMainTableJasperStream() throws Exception {
		return this.getClass().getClassLoader().getResourceAsStream("template/platform/main.jasper");
	}

	@Override
	public String getTitleName() {
		return "平台综合报表";
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	@Override
	public List<PlatformInfoReport> getMainTableReport(ReportFrequenceType reportFrequenceType,
			Map<String, Object> param, String[] resourceIds, RequestContext context, Boolean isAdmin,
			ReportConfigTemplate configTemplate, Paragraph paragraph, ReportTemplateHistory history) {
		List list = Collections.emptyList();
		try {
			/*
			 * 查询缓存的结果
			 */
			ReportHistoryResult cacheResult = reportHistoryResultRepository.query(history.getId(), paragraph.getId(),
					0L);
			if (cacheResult == null) {
				/*
				 * 缓存为空,计算数据，存入缓存
				 */
				list = platformInfoReportRepository.queryPlatformTableInfo(resourceIds, reportFrequenceType, param);
				cacheResult = new ReportHistoryResult();
				String content = JSONObject.toJSONString(list);
				cacheResult.setContent(content);
				cacheResult.setHistoryId(history.getId());
				cacheResult.setContentTemplateId(0L);
				cacheResult.setParagraphId(paragraph.getId());
				reportHistoryResultRepository.save(cacheResult);
			} else {
				list = JSONArray.parseArray(cacheResult.getContent(), PlatformInfoReport.class);
			}

		} catch (Exception e) {
			logger.error(ExceptionUtils.getFullStackTrace(e));
		}
		return list;
	}

	public ReportHistoryResultRepository getReportHistoryResultRepository() {
		return reportHistoryResultRepository;
	}
}
