package com.bocloud.cos.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.RequestContext.Catalog;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.GridHelper;
import com.bocloud.cos.entity.Paragraph;
import com.bocloud.cos.entity.ParagraphTemplateRela;
import com.bocloud.cos.entity.PublishInfo;
import com.bocloud.cos.entity.ReportConfigTemplate;
import com.bocloud.cos.intf.ReportConfigTemplateService;
import com.bocloud.cos.repository.*;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @author lyy
 */
@Service("reportConfigTemplateService")
public class ReportConfigTemplateServiceImpl implements ReportConfigTemplateService {
    private static final Logger logger = LoggerFactory.getLogger(ReportConfigTemplateServiceImpl.class);
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private ReportConfigTemplateRepository reportConfigTemplateRepository;
    @Autowired
    private ParagraphTemplateRelaRepository paragraphTemplateRelaRepository;
    @Autowired
    private PublishInfoRepository publishInfoRepository;
    @Autowired
    private ParagraphRepository paragraphRepository;
    @Autowired
    private ReportTemplateHistoryRepository reportTemplateHistoryRepository;

    @Override
    public BocloudResult list(int page, int rows, List<Param> params, Map<String, String> sorter,
                              RequestContext context) {
        GridBean gridBean;
        try {
            if (null == params) {
                params = new ArrayList<>();
            }
            if (null == sorter) {
                sorter = Maps.newHashMap();
            }
            sorter.put("gmtCreate", Common.ONE);
            Boolean isAdmin = Catalog.Tenant == context.getCatalog();
            int count = this.reportConfigTemplateRepository.count(params, context, isAdmin);
            List<ReportConfigTemplate> list = this.reportConfigTemplateRepository.list(page, rows, params, sorter, context,
                    isAdmin);
            gridBean = GridHelper.getBean(page, rows, count, list);
            return new BocloudResult(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult remove(Long reportConfigTemplateId) {
        String path = ReportConfigTemplate.class.getSimpleName() + "_" + reportConfigTemplateId;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            ReportConfigTemplate template = reportConfigTemplateRepository.query(reportConfigTemplateId);
            if (null == template) {
                logger.error("Report Config Template not exist");
                return new BocloudResult(false, "报表模板不存在");
            }
            // 删除报表模板
            this.reportConfigTemplateRepository.remove(reportConfigTemplateId);
            // 删除报表模板配置绑定的内容
            this.paragraphRepository.remove(reportConfigTemplateId);
            this.paragraphTemplateRelaRepository.remove(reportConfigTemplateId);
            this.publishInfoRepository.remove(reportConfigTemplateId);
            // 删除成功后，删除此模板所创建的所有报表
            this.reportTemplateHistoryRepository.removeByConfigTemplateId(reportConfigTemplateId);
            return new BocloudResult(true, "删除成功");
        } catch (Exception e) {
            logger.error("Remove Report Config Template fail:", e);
            return new BocloudResult(false, "删除失败");
        }
    }

    @Override
    public BocloudResult create(JSONObject basics, JSONArray contents, JSONObject subscription,
                                RequestContext context) {
        try {
            // 保存基础信息
            ReportConfigTemplate reportConfigTemplate = JSONObject.parseObject(basics.toJSONString(),
                    ReportConfigTemplate.class);

            // 判断名称是否重复
            String name = reportConfigTemplate.getName().trim();
            if (isNameExist(name, context.getTenant())) {
                return new BocloudResult(false, "该名称已经存在");
            }
            reportConfigTemplate.setCreateId(context.getTarget());
            reportConfigTemplate.setDeleted(false);
            reportConfigTemplate.setGmtCreate(new Date());
            reportConfigTemplate.setTenantId(context.getTenant());
            ReportConfigTemplate configTemplate = this.reportConfigTemplateRepository.saveReportConfigTemplate(reportConfigTemplate);
            if (null == configTemplate) {
                return new BocloudResult(false, "基础信息保存失败");
            }
            // 模板配置id
            Long id = configTemplate.getId();
            // 保存目录信息
            for (int i = 0; i < contents.size(); i++) {
                JSONObject jsonObject = contents.getJSONObject(i);
                Paragraph paragraph = JSONObject.parseObject(jsonObject.toJSONString(), Paragraph.class);
                paragraph.setConfigTemplateId(id);
                paragraph.setDeleted(false);
                Paragraph graph = this.paragraphRepository.saveParagraph(paragraph);
                if (null == graph) {
                    // 保存失败，数据库数据回滚
                    this.reportConfigTemplateRepository.remove(id);
                    return new BocloudResult(false, "保存段落信息失败");
                }
                // 保存段落与内容关系
                List<ParagraphTemplateRela> list = Lists.newArrayList();
                // 获取前端传入的报表模板
                JSONArray contentTemplates = jsonObject.getJSONArray("contentTemplates");
                if (!contentTemplates.isEmpty()) {
                    for (int j = 0; j < contentTemplates.size(); j++) {
                        Long contentTemplateId = contentTemplates.getLong(j);
                        ParagraphTemplateRela templateRela = new ParagraphTemplateRela();
                        templateRela.setParagraphId(paragraph.getId());
                        templateRela.setContentTemplateId(contentTemplateId);
                        templateRela.setConfigTemplateId(id);
                        templateRela.setDeleted(false);
                        list.add(templateRela);
                    }
                    Boolean batchSave = this.paragraphTemplateRelaRepository.batchSave(list);
                    if (batchSave.equals(false)) {
                        // 保存失败，数据库数据回显
                        this.reportConfigTemplateRepository.remove(id);
                        this.paragraphRepository.remove(id);
                        return new BocloudResult(false, "段落与内容关系保存失败");
                    }
                }
            }
            // 保存订阅信息
            PublishInfo publishInfo = JSONObject.parseObject(subscription.toJSONString(), PublishInfo.class);
            if (null != publishInfo) {
                publishInfo.setConfigTemplateId(id);
                publishInfo.setDeleted(false);
                PublishInfo info = this.publishInfoRepository.savePublishInfo(publishInfo);
                if (null == info) {
                    // 保存失败，数据库数据回显
                    this.reportConfigTemplateRepository.remove(id);
                    this.paragraphRepository.remove(id);
                    this.paragraphTemplateRelaRepository.remove(id);
                    return new BocloudResult(false, "保存订阅信息失败");
                }
            }
            return new BocloudResult(true, "保存成功");
        } catch (Exception e) {
            logger.error("create Report Config Template fail:", e);
            return new BocloudResult(false, "保存失败");
        }

    }

    private boolean isNameExist(String name, Long tenantId) {
        try {
            ReportConfigTemplate configTemplate = this.reportConfigTemplateRepository.queryByName(name, tenantId);
            return configTemplate != null;
        } catch (Exception e) {
            logger.error("查询名称失败", e);
        }
        return true;
    }

    @Override
    public BocloudResult update(Long id, Boolean status, Long userId) {
        String path = ReportConfigTemplate.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            ReportConfigTemplate template = reportConfigTemplateRepository.query(id);
            if (null == template) {
                logger.error("Report Config Template not exist");
                return new BocloudResult(false, "报表模板不存在");
            }
            template.setStatus(status);
            template.setGmtModify(new Date());
            template.setMenderId(userId);
            // 修改报表模板状态
            reportConfigTemplateRepository.update(template);
            return new BocloudResult(true, "更新状态成功");
        } catch (Exception e) {
            logger.error("update Report Config Template fail:", e);
            return new BocloudResult(false, "更新状态失败");
        }
    }

    @Override
    public BocloudResult detail(Long id) {
        try {
            ReportConfigTemplate configTemplate = this.reportConfigTemplateRepository.detail(id);
            if (null != configTemplate) {
                return new BocloudResult(true, configTemplate, "查询成功");
            } else {
                return new BocloudResult(false, configTemplate, "查询报表配置详情，数据为空");
            }
        } catch (Exception e) {
            logger.error("Query Report Config detail fail:", e);
            return new BocloudResult(false, "查询报表配置明细失败");
        }
    }
}
