package com.bocloud.cos.service;

import com.bocloud.common.model.*;
import com.bocloud.common.model.RequestContext.Catalog;
import com.bocloud.common.utils.GridHelper;
import com.bocloud.cos.intf.ReportResourceService;
import com.bocloud.cos.model.ReportResource;
import com.bocloud.cos.repository.ReportResourceRepository;
import com.bocloud.cos.service.internal.SmsUserService;
import com.bocloud.internel.entity.User;
import com.google.common.collect.Maps;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @author lyy
 */
@Service("reportResourceService")
public class ReportResourceServiceImpl implements ReportResourceService {
    Logger logger = LoggerFactory.getLogger(this.getClass().getName());
    @Autowired
    private ReportResourceRepository reportResourceRepository;
    @Autowired
    private SmsUserService smsUserService;

    @Override
    public BocloudResult list(Integer page, Integer rows, List<Param> params, Map<String, String> sorterMap,
                              String reportCategory, RequestContext context) {
        GridBean gridBean;
        try {
            if (null == params) {
                params = new ArrayList<>();
            }
            if (null == sorterMap) {
                sorterMap = Maps.newHashMap();
            }
            // sorterMap.put("gmtCreate", Common.ONE);
            Long userId = context.getTarget();
            BocloudResult<User> result = smsUserService.detail(userId);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            if (null == result.getData()) {
                logger.warn("user does not exists");
                return new BocloudResult(false, "用户存在问题，请联系管理员");
            }
            Boolean isAdmin = Catalog.Tenant == context.getCatalog();
            boolean isCmp = true;
            if (0 != context.getTenant()) {
                isCmp = false;
            }
            int count = this.reportResourceRepository.count(params, reportCategory, context, isAdmin, isCmp, null);
            List<ReportResource> list = this.reportResourceRepository.list(page, rows, params, sorterMap, reportCategory,
                    context, isAdmin, isCmp, null);
            gridBean = GridHelper.getBean(page, rows, count, list);
            return new BocloudResult(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error(ExceptionUtils.getFullStackTrace(e));
            return new BocloudResult(false, "查询失败");
        }
    }
}
