package com.bocloud.cos.service.internal;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.enums.BoCloudMethod;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.service.utils.CaptchaCache;
import com.bocloud.internel.entity.*;
import com.bocloud.service.core.CurrentService;
import com.bocloud.service.core.Service;
import com.bocloud.service.http.core.ServiceFactory;
import com.bocloud.service.http.model.RemoteService;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * cmp内部访问
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年1月9日
 */
@Component("imsInternalService")
public class ImsInternalService implements InitializingBean {

    private static Logger logger = LoggerFactory.getLogger(ImsInternalService.class);

    @Autowired
    private ServiceFactory serviceFactory;
    @Autowired
    private CurrentService currentService;
    @Autowired
    private CaptchaCache captchaCache;
    private final Map<String, Object> headers = new HashMap<>(4);
    private static final Service SERVICE = Service.create(BoCloudService.IMS);
    private List<CloudServer> cloudServers = null;
    private LocalDateTime requestVmTime = null;
    private List<Server> servers = null;
    private LocalDateTime requestServerTime = null;
    private List<Cluster> clusters = null;
    private LocalDateTime requestClusetrTime = null;

//    @Scheduled(cron = "${screen.task.data:0 */10 * * * ?}")
//    protected void syncScreenInfo() throws Exception {
//        try {
//            List<Server> listHosts = this.listHosts();
//            captchaCache.putCacheMin("listHosts", JSON.toJSONString(listHosts));
//        } catch (Exception e) {
//            logger.error("定時任务查询主机异常：" + e.getMessage());
//        }
//        try {
//            if(!StringUtils.isEmpty(apiKey)) {
//                RequestContext context = new RequestContext();
//                context.setApiKey(apiKey);
//                List<CloudServer> listVmsByVendor = listVmsByVendor(null, false, context );
//                captchaCache.putCacheMin("listVmsByVendor", JSON.toJSONString(listVmsByVendor));
//            }
//        } catch (Exception e) {
//            logger.error("定時任务查询虛機异常：" + e.getMessage());
//        }
//        try {
//            List<Cluster> listClustersByType = listClustersByType(null);
//            captchaCache.putCacheMin("listClustersByType", JSON.toJSONString(listClustersByType));
//        } catch (Exception e) {
//            logger.error("定時任务查询虛機异常：" + e.getMessage());
//        }
//        try {
//            List<CloudVendor> listCloudVendor = listCloudVendor(null);
//            captchaCache.putCacheMin("listCloudVendor", JSON.toJSONString(listCloudVendor));
//        } catch (Exception e) {
//            logger.error("查询云平台数据异常：" + e.getMessage());
//        }
//    }

    public BocloudResult recycle(String uuid) {
        Map<String, Object> paramMap = MapTools.simpleMap("action", "recycleByUuid");
        JSONObject params = new JSONObject();
        params.put("uuid", uuid);
        paramMap.put("params", params.toJSONString());
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/vms/0", BoCloudMethod.PATCH, headers, paramMap);
        return remoteService.invoke();
    }

    public CloudServer queryBySn(String uuid) {
        JSONObject paramMap = new JSONObject();
        paramMap.put("condition", "uuid");
        paramMap.put("uuid", uuid);
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/vms/condition", BoCloudMethod.GET, headers,
                MapTools.simpleMap("condition", paramMap.toJSONString()));
        BocloudResult<?> invoke = remoteService.invoke();
        return JSONObject.parseObject(JSONObject.toJSONString(invoke.getData()), CloudServer.class);
    }

    public List<Volume> listByVmId(Long serverId) {
        JSONObject object = new JSONObject();
        object.put(Common.CONDITION, "listByVmId");
        object.put("serverId", serverId);
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/volumes/condition", BoCloudMethod.GET, headers,
                MapTools.simpleMap(Common.CONDITION, object.toJSONString()));
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return null;
        }
        return JSONArray.parseArray(JSONObject.toJSONString(result.getData()), Volume.class);
    }

    public Volume listVolumeByUUId(String uuid) {
        ArrayList<Param> params = Lists.newArrayList(new Param(MapTools.simpleMap("uuid", uuid), Sign.EQ));
        Pager pager = new Pager(1, Integer.MAX_VALUE, params, null);
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/volumes/condition", BoCloudMethod.GET, headers, pager.toMap());
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return null;
        }
        JSONObject volumePagerObjects = JSONObject.parseObject(JSONObject.toJSONString(result.getData()));
        JSONArray rows = volumePagerObjects.getJSONArray("rows");
        List<Volume> volumes = JSONArray.parseArray(rows.toJSONString(), Volume.class);
        return null == volumes || volumes.isEmpty() ? null : volumes.get(0);
    }

    public BocloudResult generateName(Long nameRuleId, JSONObject params) {
        RemoteService remoteService = serviceFactory
                .build(SERVICE, "/v1/namings/" + nameRuleId, BoCloudMethod.PATCH, headers,
                        MapTools.simpleMap(Common.PARAMS, params));
        return remoteService.invoke();
    }

    public BocloudResult getPool(Long vendorId, Long tenantId) {
        JSONObject object = new JSONObject();
        object.put(Common.CONDITION, "listByVendorId");
        object.put("vendorId", vendorId);
        object.put("tenantId", tenantId);
        RemoteService remoteService = serviceFactory
                .build(SERVICE, "/v1/pool/groups/condition", BoCloudMethod.GET, headers,
                        MapTools.simpleMap(Common.CONDITION, object.toJSONString()));
        return remoteService.invoke();
    }

    public PoolGroup getPoolGroup(Long poolGroupId) {
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/pool/groups/" + poolGroupId, BoCloudMethod.GET, headers, null);
        BocloudResult<?> invoke = remoteService.invoke();
        PoolGroup poolGroup = JSONObject.parseObject(JSONObject.toJSONString(invoke.getData()), PoolGroup.class);
        return poolGroup;
    }

    public BocloudResult serverDetail(String apiKey, Long serverId) {
        headers.put(Common.API_KEY, apiKey);
        JSONObject object = new JSONObject();
        object.put("serverId", serverId);
        object.put(Common.CONDITION, "basic");
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/vms/condition", BoCloudMethod.GET, headers,
                MapTools.simpleMap(Common.CONDITION, object.toJSONString()));
        return remoteService.invoke();
    }

    public BocloudResult delete(List<String> instanceIds) {
        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put("action", "removeByInstanceIds");
        paramMap.put(Common.PARAMS, JSONObject.toJSONString(instanceIds));
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/vms", BoCloudMethod.PATCH, headers, paramMap);
        return remoteService.invoke();
    }

    public Map<String, String> tenantProvince() {
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/dictionaries", BoCloudMethod.GET, headers,
                MapTools.simpleMap("value", "TENANT_PROVINCE"));
        BocloudResult invoke = remoteService.invoke();
        Map<String, String> result = new HashMap<>();
        if (invoke.getData() != null) {
            List<Dictionary> dictionaries =
                    JSONArray.parseArray(JSONObject.toJSONString(invoke.getData()), Dictionary.class);
            for (Dictionary dictionary : dictionaries) {
                result.put(dictionary.getValue(), dictionary.getName());
            }
        }
        return result;
    }

    /**
     * 获取云平台列表
     *
     * @return
     */
    public List<CloudVendor> getVendorList() {
        ArrayList<Param> params = new ArrayList<>();
        params.add(new Param(MapTools.simpleMap("vendorCategory", "COMPUTE"), Sign.EQ));
        Pager pager = new Pager(1, 9999, params, null);
        Map<String, Object> requestMap = pager.toMap();
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/vendors", BoCloudMethod.GET, headers, requestMap);
        BocloudResult invoke = remoteService.invoke();
        Assert.notNull(invoke.getData(), "获取平台列表失败");
        JSONObject vendorPager = JSONObject.parseObject(JSONObject.toJSONString(invoke.getData()));
        JSONArray rows = vendorPager.getJSONArray("rows");
        return JSONArray.parseArray(rows.toJSONString(), CloudVendor.class);
    }

    /**
     * 获取云平台信息
     *
     * @return
     */
    public CloudVendor getVendorDetail(Long id) {
        ArrayList<Param> params = new ArrayList<>();
        Map<String, Object> paramsMap = MapTools.simpleMap("vendorCategory", "COMPUTE");
        paramsMap.put("id", id);
        params.add(new Param(paramsMap, Sign.EQ));
        Pager pager = new Pager(1, 9999, params, null);
        Map<String, Object> requestMap = pager.toMap();
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/vendors", BoCloudMethod.GET, headers, requestMap);
        BocloudResult invoke = remoteService.invoke();
        Assert.notNull(invoke.getData(), "获取平台信息失败");
        JSONObject vendorPager = JSONObject.parseObject(JSONObject.toJSONString(invoke.getData()));
        JSONArray rows = vendorPager.getJSONArray("rows");
        List<CloudVendor> cloudVendors = JSONArray.parseArray(rows.toJSONString(), CloudVendor.class);
        Assert.isTrue(cloudVendors.size() <= 1, "more than one vendor in the list");
        return cloudVendors.isEmpty() ? null : cloudVendors.get(0);
    }

    /**
     * 获取主机列表
     *
     * @return
     */
    public BocloudResult getHostList(Long vendorId, Long vdcId, Long clusterId, String name) {
        return this.getHostList(vendorId, vdcId, clusterId, name, null);
    }

    /**
     * 获取主机列表
     *
     * @return
     */
    public BocloudResult getHostList(Long vendorId, Long vdcId, Long clusterId, String name, String vendorType) {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put(Common.CONDITION, "listByCondition");
        jsonObject.put("vendorId", vendorId);
        jsonObject.put("vendorType", vendorType);
        jsonObject.put("vdcId", vdcId);
        jsonObject.put("clusterId", clusterId);
        jsonObject.put("name", name);
        Map<String, Object> requestMap = MapTools.simpleMap(Common.CONDITION, jsonObject);
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/hosts/condition", BoCloudMethod.GET, headers, requestMap);
        return remoteService.invoke();
    }

    /**
     * 获取集群信息
     *
     * @param page
     * @param rows
     * @param params
     * @param sorter
     * @return
     */
    public BocloudResult listClusters(int page, int rows, List<Param> params, Map<String, String> sorter) {
        Pager pager = new Pager(page, rows, params, sorter);
        RemoteService remote = serviceFactory.build(SERVICE, "/v1/clusters", BoCloudMethod.GET, headers, pager.toMap());
        BocloudResult result = remote.invoke();
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        Assert.notNull(result.getData(), "获取集群信息失败！");
        return result;
    }

    public List<Cluster> listClustersByType(String type) {
        List<Cluster> listClustersByType = JSON.parseArray(captchaCache.getCache("listClustersByType"), Cluster.class);
        if (ListTool.isEmpty(listClustersByType)) {
            Pager pager = new Pager(1, 9999, new ArrayList<>(), null);
            Map<String, Object> requestMap = pager.toMap();
            RemoteService remoteService =
                    serviceFactory.build(SERVICE, "/v1/clusters", BoCloudMethod.GET, headers, requestMap);
            BocloudResult result = remoteService.invoke();
            if (result.isFailed()) {
                return new ArrayList<>();
            }
            String content = JSONObject.toJSONString(result.getData());
            if (org.springframework.util.StringUtils.isEmpty(content)) {
                return new ArrayList<>();
            }
            GridBean gridBean = JSONObject.parseObject(content, GridBean.class);
            listClustersByType = JSONArray.parseArray(JSONObject.toJSONString(gridBean.getRows()), Cluster.class);
            captchaCache.putCacheMin("listClustersByType", JSON.toJSONString(listClustersByType));
        }
        if (!StringUtils.isEmpty(type)) {
            listClustersByType = listClustersByType.stream().filter(cluster -> type.equalsIgnoreCase(cluster.getType()))
                    .collect(Collectors.toList());
        }
        return listClustersByType;
    }

    public List<Server> listHosts() {
        List<Server> listHosts = JSON.parseArray(captchaCache.getCache("listHosts"), Server.class);
        if (ListTool.isEmpty(listHosts)) {
            List<Param> params = new ArrayList<>();
            Pager pager = new Pager(1, 9999, params, null);
            Map<String, Object> requestMap = pager.toMap();
            RemoteService remoteService =
                    serviceFactory.build(SERVICE, "/v1/hosts", BoCloudMethod.GET, headers, requestMap);
            BocloudResult result = remoteService.invoke();
            if (result.isFailed()) {
                return new ArrayList<>();
            }
            String content = JSONObject.toJSONString(result.getData());
            if (StringUtils.isEmpty(content)) {
                return new ArrayList<>();
            }
            GridBean gridBean = JSONObject.parseObject(content, GridBean.class);
            listHosts = JSONArray.parseArray(JSONObject.toJSONString(gridBean.getRows()), Server.class);
            captchaCache.putCacheMin("listHosts", JSON.toJSONString(listHosts));
        }
        return listHosts;
    }

    /**
     * 获取虚机列表
     *
     * @return
     */
    public BocloudResult getVmList(Long vendorId, Long vdcId, Long clusterId) {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put(Common.CONDITION, "listByCondition");
        jsonObject.put("vendorId", vendorId);
        jsonObject.put("vdcId", vdcId);
        jsonObject.put("clusterId", clusterId);
        Map<String, Object> requestMap = MapTools.simpleMap(Common.CONDITION, jsonObject);
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/vms/condition", BoCloudMethod.GET, headers, requestMap);
        return remoteService.invoke();
    }

    /**
     * 获取虚机列表
     *
     * @param serverIds
     * @return
     */
    public BocloudResult getVmList(List<Long> serverIds) {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put(Common.CONDITION, "listByVms");
        jsonObject.put("serverIds", serverIds);
        Map<String, Object> requestMap = MapTools.simpleMap(Common.CONDITION, jsonObject);
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/vms/condition", BoCloudMethod.GET, headers, requestMap);
        return remoteService.invoke();
    }

    /**
     * 获取虚机列表
     *
     * @param tenantIds
     * @return
     */
    public List<CloudServer> listVms(List<Long> tenantIds, Boolean isAll, List<Long> projectIds) {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put(Common.CONDITION, "listTenantProject");
        if (!ListTool.isEmpty(tenantIds)) {
            jsonObject.put("tenantIds", tenantIds);
        }
        if (!ListTool.isEmpty(projectIds)) {
            jsonObject.put("projectIds", projectIds);
        }
        jsonObject.put("isAll", isAll);
        Map<String, Object> requestMap = MapTools.simpleMap(Common.CONDITION, jsonObject);
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/vms/condition", BoCloudMethod.GET, headers, requestMap);
        BocloudResult<?> result = remoteService.invoke();
        if (result.isFailed() || null == result.getData()) {
            return new ArrayList<>();
        }
        cloudServers = JSONArray.parseArray(JSONObject.toJSONString(result.getData()), CloudServer.class);
        requestVmTime = LocalDateTime.now();
        return cloudServers;
    }

    /**
     * 获取云硬盘
     *
     * @param vendorId
     * @return
     */
    public BocloudResult getVolume(Long vendorId) {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("condition", "listByCondition");
        jsonObject.put("vendorId", vendorId);
        Map<String, Object> requestMap = MapTools.simpleMap(Common.CONDITION, jsonObject);
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/volumes/condition", BoCloudMethod.GET, headers, requestMap);
        return remoteService.invoke();
    }

    /**
     * 获取数据存储
     *
     * @param vendorId
     * @return
     */
    public BocloudResult getDataStore(Long vendorId, Long vdcId) {
        ArrayList<Param> params = new ArrayList<>();
        if (null != vendorId && 0L != vendorId) {
            params.add(new Param(MapTools.simpleMap("vendorId", vendorId), Sign.EQ));
        }
        if (null != vdcId && 0L != vdcId) {
            params.add(new Param(MapTools.simpleMap("vdcId", vdcId), Sign.EQ));
        }
        Pager pager = new Pager(1, 9999, params, null);
        Map<String, Object> requestMap = pager.toMap();
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/datastores", BoCloudMethod.GET, headers, requestMap);
        return remoteService.invoke();
    }

    /**
     * 获取浮动Ip
     *
     * @param vendorId
     * @return
     */
    public BocloudResult getFloatIp(Long vendorId) {
        ArrayList<Param> params = new ArrayList<>();
        if (null != vendorId && 0L != vendorId) {
            params.add(new Param(MapTools.simpleMap("vendorId", vendorId), Sign.EQ));
        }
        Pager pager = new Pager(1, 9999, params, null);
        Map<String, Object> requestMap = pager.toMap();
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/floatips", BoCloudMethod.GET, headers, requestMap);
        return remoteService.invoke();
    }

    public List<CloudServer> listServers(RequestContext context) {
        headers.put(Common.API_KEY, context.getApiKey());
        List<Param> params = new ArrayList<>();
        params.add(new Param(MapTools.simpleMap("isRecycle", 1), Sign.EQ));
        Pager pager = new Pager(1, 9999, params, null);
        Map<String, Object> requestMap = pager.toMap();
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/vms", BoCloudMethod.GET, headers, requestMap);
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new ArrayList<>();
        }
        String content = JSONObject.toJSONString(result.getData());
        if (!StringUtils.isNotEmpty(content)) {
            return new ArrayList<>();
        }
        GridBean gridBean = JSONObject.parseObject(content, GridBean.class);
        return JSONArray.parseArray(JSONObject.toJSONString(gridBean.getRows()), CloudServer.class);
    }

    /**
     * 获取已使用Ip
     *
     * @param poolName
     * @return
     */
    public BocloudResult getIpNumber(String poolName) {
        ArrayList<Param> params = new ArrayList<>();
        if (poolName != null) {
            params.add(new Param(MapTools.simpleMap("poolName", poolName), Sign.LK));
        }
        Pager pager = new Pager(1, 9999, params, null);
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/ips", BoCloudMethod.GET, headers, pager.toMap());
        return remoteService.invoke();
    }

    /**
     * 获取ip池列表
     *
     * @param poolName
     * @return
     */
    public BocloudResult getIpPoolList(String poolName) {
        ArrayList<Param> params = new ArrayList<>();
        params.add(new Param(MapTools.simpleMap("countIp", true), Sign.EQ));
        if (null != poolName) {
            params.add(new Param(MapTools.simpleMap("name", poolName), Sign.LK));
        }
        Pager pager = new Pager(1, 9999, params, null);
        Map<String, Object> requestMap = pager.toMap();
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/ip/pools", BoCloudMethod.GET, headers, requestMap);
        return remoteService.invoke();
    }

    /**
     * ip池详情
     *
     * @param poolId
     * @return
     */
    public BocloudResult getIpPool(Long poolId) {
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/ip/pools/" + poolId, BoCloudMethod.GET, headers, null);
        return remoteService.invoke();
    }

    // 大屏相关添加接口

    /**
     * 获取数据中心列表
     *
     * @param context
     * @return
     */
    public List<DataCenter> listDataCenters(RequestContext context) {
        headers.put(Common.API_KEY, context.getApiKey());
        List<Param> params = new ArrayList<>();
        Pager pager = new Pager(1, 9999, params, null);
        Map<String, Object> requestMap = pager.toMap();
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/dcs", BoCloudMethod.GET, headers, requestMap);
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new ArrayList<>();
        }
        String content = JSONObject.toJSONString(result.getData());
        if (!StringUtils.isNotEmpty(content)) {
            return new ArrayList<>();
        }
        GridBean gridBean = JSONObject.parseObject(content, GridBean.class);
        return JSONArray.parseArray(JSONObject.toJSONString(gridBean.getRows()), DataCenter.class);
    }

    public List<Vdc> listVDCs(Long id, RequestContext requestContext) {
        headers.put(Common.API_KEY, requestContext.getApiKey());
        List<Param> params = new ArrayList<>();
        if (id != null && id != 0L) {
            Map<String, Object> paramMap = MapTools.simpleMap("id", id);
            params.add(new Param(paramMap, Sign.EQ));
        }
        Pager pager = new Pager(1, 9999, params, null);
        Map<String, Object> requestMap = pager.toMap();
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/vdcs", BoCloudMethod.GET, headers, requestMap);
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new ArrayList<>();
        }
        String content = JSONObject.toJSONString(result.getData());
        if (!StringUtils.isNotEmpty(content)) {
            return new ArrayList<>();
        }
        GridBean gridBean = JSONObject.parseObject(content, GridBean.class);
        return JSONArray.parseArray(JSONObject.toJSONString(gridBean.getRows()), Vdc.class);
    }

    public List<Vdc> listVDCsByVendor(List<Long> vendorIds, RequestContext requestContext) {
        headers.put(Common.API_KEY, requestContext.getApiKey());
        List<Param> params = new ArrayList<>();
        if (vendorIds != null && !vendorIds.isEmpty()) {
            String jsonString = JSONObject.toJSONString(vendorIds);
            Map<String, Object> paramMap =
                    MapTools.simpleMap("vendorId", jsonString.substring(1, jsonString.length() - 1));
            params.add(new Param(paramMap, Sign.IN));
        }
        Pager pager = new Pager(1, 9999, params, null);
        Map<String, Object> requestMap = pager.toMap();
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/vdcs", BoCloudMethod.GET, headers, requestMap);
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new ArrayList<>();
        }
        String content = JSONObject.toJSONString(result.getData());
        if (!StringUtils.isNotEmpty(content)) {
            return new ArrayList<>();
        }
        GridBean gridBean = JSONObject.parseObject(content, GridBean.class);
        return JSONArray.parseArray(JSONObject.toJSONString(gridBean.getRows()), Vdc.class);
    }

    public DataCenter getDcDetail(Long dcId, RequestContext requestContext) {
        headers.put(Common.API_KEY, requestContext.getApiKey());
        List<Param> params = new ArrayList<>();
        Pager pager = new Pager(1, 9999, params, null);
        Map<String, Object> requestMap = pager.toMap();
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/dcs/" + dcId, BoCloudMethod.GET, headers, requestMap);
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return null;
        }
        return JSONObject.parseObject(JSONObject.toJSONString(result.getData()), DataCenter.class);
    }

    /**
     * 获取机房列表
     *
     * @param context
     * @return
     */
    public List<Room> listRooms(Long vdcId, RequestContext context) {
        headers.put(Common.API_KEY, context.getApiKey());
        List<Param> params = new ArrayList<>();
        if (vdcId != null && vdcId != 0L) {
            Map<String, Object> paramMap = MapTools.simpleMap("vdcId", vdcId);
            params.add(new Param(paramMap, Sign.EQ));
        }
        Pager pager = new Pager(1, 9999, params, null);
        Map<String, Object> requestMap = pager.toMap();
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/rooms", BoCloudMethod.GET, headers, requestMap);
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new ArrayList<>();
        }
        String content = JSONObject.toJSONString(result.getData());
        if (!StringUtils.isNotEmpty(content)) {
            return new ArrayList<>();
        }
        GridBean gridBean = JSONObject.parseObject(content, GridBean.class);
        return JSONArray.parseArray(JSONObject.toJSONString(gridBean.getRows()), Room.class);
    }

    /**
     * 获取指定数据中心下虚拟机列表
     *
     * @param context
     * @return
     */
    public List<CloudServer> listVdcVms(Long vdcId, RequestContext context) {
        headers.put(Common.API_KEY, context.getApiKey());
        List<Param> params = new ArrayList<>();
        Map<String, Object> paramMap = MapTools.simpleMap("isTemplate", false);
        if (vdcId != null && vdcId != 0L) {
            paramMap.put("vdcId", vdcId);
        }
        params.add(new Param(paramMap, Sign.EQ));
        Pager pager = new Pager(1, 9999, params, null);
        Map<String, Object> requestMap = pager.toMap();
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/vms", BoCloudMethod.GET, headers, requestMap);
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new ArrayList<>();
        }
        String content = JSONObject.toJSONString(result.getData());
        if (!StringUtils.isNotEmpty(content)) {
            return new ArrayList<>();
        }
        GridBean gridBean = JSONObject.parseObject(content, GridBean.class);
        return JSONArray.parseArray(JSONObject.toJSONString(gridBean.getRows()), CloudServer.class);
    }

    /**
     * 获取指定类型虚拟机列表
     *
     * @param context
     * @return
     */
    private static String apiKey = "";

    public List<CloudServer> listVmsByVendor(String vendorType, Boolean isAll, RequestContext context) {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put(Common.CONDITION, "listTenantProject");
        jsonObject.put("isAll", isAll);
        Map<String, Object> requestMap = MapTools.simpleMap(Common.CONDITION, jsonObject);
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/vms/condition", BoCloudMethod.GET, headers, requestMap);
        BocloudResult<?> result = remoteService.invoke();
        if (result.isFailed() || null == result.getData()) {
            return new ArrayList<>();
        }
        List<CloudServer> listVmsByVendor = JSONArray.parseArray(JSONObject.toJSONString(result.getData()), CloudServer.class);
        captchaCache.putCache("listVmsByVendor", JSON.toJSONString(listVmsByVendor));
        if (!StringUtils.isEmpty(vendorType)) {
            listVmsByVendor = listVmsByVendor.stream().filter(
                    cloudServer -> cloudServer.getVendorType().equals(vendorType))
                    .collect(Collectors.toList());
        }
        return listVmsByVendor;
    }

    /**
     * 获取指定类型虚拟机列表
     *
     * @param context
     * @return
     */
    public List<CloudServer> listVmsByDcId(Long dcId, Boolean isAll, RequestContext context) {
        headers.put(Common.API_KEY, context.getApiKey());
        JSONObject jsonObject = new JSONObject();
        List<Long> dcVendorIds = null;
        if (null != dcId && 0L != dcId) {
            List<CloudVendor> cloudVendors = this.listCloudVendor(null);
            dcVendorIds =
                    cloudVendors.stream().filter(cloudVendor -> dcId.equals(cloudVendor.getDcId())).map(CloudVendor::getId)
                            .collect(Collectors.toList());
        }
        jsonObject.put(Common.CONDITION, "listTenantProject");
        jsonObject.put("isAll", isAll);
        Map<String, Object> requestMap = MapTools.simpleMap(Common.CONDITION, jsonObject);
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/vms/condition", BoCloudMethod.GET, headers, requestMap);
        BocloudResult<?> result = remoteService.invoke();
        if (result.isFailed() || null == result.getData()) {
            return new ArrayList<>();
        }
        cloudServers = JSONArray.parseArray(JSONObject.toJSONString(result.getData()), CloudServer.class);
        requestVmTime = LocalDateTime.now();
        List<Long> finalDcVendorIds = dcVendorIds;
        return cloudServers.stream()
                .filter(cloudServer -> null == finalDcVendorIds || finalDcVendorIds.contains(cloudServer.getVendorId()))
                .collect(Collectors.toList());
    }

    /**
     * 获取指定集群下虚拟机列表
     *
     * @param context
     * @return
     */
    public List<CloudServer> listClusterVms(Long clusterId, RequestContext context) {
        headers.put(Common.API_KEY, context.getApiKey());
        List<Param> params = new ArrayList<>();
        Map<String, Object> paramMap = MapTools.simpleMap("isTemplate", false);
        if (clusterId != null && clusterId != 0L) {
            paramMap.put("clusterId", clusterId);
        }
        params.add(new Param(paramMap, Sign.EQ));
        Pager pager = new Pager(1, 9999, params, null);
        Map<String, Object> requestMap = pager.toMap();
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/vms", BoCloudMethod.GET, headers, requestMap);
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new ArrayList<>();
        }
        String content = JSONObject.toJSONString(result.getData());
        if (!StringUtils.isNotEmpty(content)) {
            return new ArrayList<>();
        }
        GridBean gridBean = JSONObject.parseObject(content, GridBean.class);
        return JSONArray.parseArray(JSONObject.toJSONString(gridBean.getRows()), CloudServer.class);
    }

    /**
     * 获取云平台列表
     *
     * @param context
     * @return
     */
    public List<CloudVendor> listVendors(RequestContext context) {
        headers.put(Common.API_KEY, context.getApiKey());
        List<Param> params = new ArrayList<>();
        params.add(new Param(MapTools.simpleMap("vendorCategory", "COMPUTE"), Sign.EQ));
        Pager pager = new Pager(1, 9999, params, null);
        Map<String, Object> requestMap = pager.toMap();
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/vendors", BoCloudMethod.GET, headers, requestMap);
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new ArrayList<>();
        }
        String content = JSONObject.toJSONString(result.getData());
        if (!StringUtils.isNotEmpty(content)) {
            return new ArrayList<>();
        }
        GridBean gridBean = JSONObject.parseObject(content, GridBean.class);
        return JSONArray.parseArray(JSONObject.toJSONString(gridBean.getRows()), CloudVendor.class);
    }

    public List<CloudVendor> listCloudVendor(String type) {

        List<Param> params = new ArrayList<>();
        Map<String, Object> paramMap = MapTools.simpleMap("vendorCategory", "COMPUTE");
        params.add(new Param(paramMap, Sign.EQ));
        Pager pager = new Pager(1, 9999, params, null);
        Map<String, Object> requestMap = pager.toMap();
        Map<String, Object> headerMap = new HashMap<>();
        headerMap.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        headerMap.put("source", currentService.getService().getName());
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/vendors", BoCloudMethod.GET, headerMap, requestMap);
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new ArrayList<>();
        }
        String content = JSONObject.toJSONString(result.getData());
        if (org.springframework.util.StringUtils.isEmpty(content)) {
            return new ArrayList<>();
        }
        GridBean gridBean = JSONObject.parseObject(content, GridBean.class);
        List<CloudVendor> cloudVendors = JSONArray.parseArray(JSONObject.toJSONString(gridBean.getRows()), CloudVendor.class);
        captchaCache.putCache("listCloudVendor", JSON.toJSONString(cloudVendors));

        if (!StringUtils.isEmpty(type)) {
            cloudVendors =
                    cloudVendors.stream()
                            .filter(cloudVendor -> type.equalsIgnoreCase(cloudVendor.getType()))
                            .collect(Collectors.toList());
        }
        return cloudVendors;
    }

    /**
     * 获取集群列表
     *
     * @param context
     * @return
     */
    public List<Cluster> listClusters(Long vdcId, RequestContext context) {
        headers.put(Common.API_KEY, context.getApiKey());
        List<Param> params = new ArrayList<>();
        if (0L != vdcId && null != vdcId) {
            params.add(new Param(MapTools.simpleMap("vdcId", vdcId), Sign.EQ));
        }
        Pager pager = new Pager(1, 9999, params, null);
        Map<String, Object> requestMap = pager.toMap();
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/clusters", BoCloudMethod.GET, headers, requestMap);
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new ArrayList<>();
        }
        String content = JSONObject.toJSONString(result.getData());
        if (!StringUtils.isNotEmpty(content)) {
            return new ArrayList<>();
        }
        GridBean gridBean = JSONObject.parseObject(content, GridBean.class);
        return JSONArray.parseArray(JSONObject.toJSONString(gridBean.getRows()), Cluster.class);
    }

    public List<CloudVendor> listCloudVendor(String type, RequestContext context) {
        List<Param> params = new ArrayList<>();
        Map<String, Object> paramMap = MapTools.simpleMap("vendorCategory", "COMPUTE");
        if (null != type && !type.isEmpty()) {
            paramMap.put("type", type);
        }
        params.add(new Param(paramMap, Sign.EQ));
        Pager pager = new Pager(1, 9999, params, null);
        Map<String, Object> requestMap = pager.toMap();
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/vendors", BoCloudMethod.GET, headers, requestMap);
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new ArrayList<>();
        }
        String content = JSONObject.toJSONString(result.getData());
        if (!StringUtils.isNotEmpty(content)) {
            return new ArrayList<>();
        }
        GridBean gridBean = JSONObject.parseObject(content, GridBean.class);
        return JSONArray.parseArray(JSONObject.toJSONString(gridBean.getRows()), CloudVendor.class);
    }

    public List<NetworkCard> listNetworkCards(Long templateId) {
        List<Param> params = new ArrayList<>();
        Map<String, Object> paramMap = MapTools.simpleMap("hostId", templateId);
        paramMap.put("category", "VM");
        params.add(new Param(paramMap, Sign.EQ));
        Pager pager = new Pager(1, 9999, params, null);
        Map<String, Object> requestMap = pager.toMap();
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/nics", BoCloudMethod.GET, headers, requestMap);
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new ArrayList<>();
        }
        String content = JSONObject.toJSONString(result.getData());
        if (!StringUtils.isNotEmpty(content)) {
            return new ArrayList<>();
        }
        GridBean gridBean = JSONObject.parseObject(content, GridBean.class);
        return JSONArray.parseArray(JSONObject.toJSONString(gridBean.getRows()), NetworkCard.class);
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        this.headers.put("source", currentService.getService().getName());
    }

    public BocloudResult selectAllByVendorId(Long vendorId) {
        Map<String, Object> requestMap = MapTools.simpleMap("vendorId", vendorId);
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/vms/selectAllByVendorId", BoCloudMethod.GET, headers, requestMap);
        return remoteService.invoke();
    }
}
