package com.bocloud.cos.service.jasper.handler.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cos.common.ReportConstants;
import com.bocloud.cos.entity.*;
import com.bocloud.cos.enums.ReportFrequenceType;
import com.bocloud.cos.intf.MonitorReportService;
import com.bocloud.cos.intf.ReportCreateService;
import com.bocloud.cos.model.GraphData;
import com.bocloud.cos.repository.ReportHistoryResultRepository;
import com.bocloud.cos.service.MonitorReportServiceImpl;
import com.bocloud.cos.service.jasper.utils.XyLineChartReportBuilder;
import net.sf.jasperreports.engine.JasperPrint;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Collections;
import java.util.List;
import java.util.Map;

/**
 * @author hufy
 */
public class MonitorJasperHandler extends AbstractJasperHandler {

    private static Logger logger = LoggerFactory.getLogger(MonitorJasperHandler.class);

    public static final String CPU_USEAGE_LINE_CHART_NAME = "CPU使用率";

    public static final String RAM_USE_LINE_CHART_NAME = "内存使用量";

    public static final String DISK_USAGE_LINE_CHART_NAME = "磁盘使用率";

    public static final String DISK_IO_LINE_CHART_NAME = "磁盘IO";

    public static final String NET_IO_LINE_CHART_NAME = "网络速率";

    public static final String SYSTEM_LOAD_CHART_NAME = "系统LOAD";

    ReportHistoryResultRepository reportHistoryResultRepository;

    public MonitorJasperHandler(ReportCreateService service) {
        super(service);
        this.reportHistoryResultRepository = ((MonitorReportServiceImpl) this.getService()).getReportHistoryResultRepository();
    }

    @Override
    public JasperPrint getJasperPrint(ReportTemplateHistory history, Map<String, Object> params,
                                      ContentTemplate contentTemplate, ReportFrequenceType frequenceType, String[] resourceIds,
                                      Paragraph paragraph, ReportConfigTemplate configTemplate) {
        MonitorReportService monitorReportService = (MonitorReportService) getService();
        JasperPrint jasperPrint = null;
        String chartName = (String) params.get(ReportConstants.CHART_NAME);
        List<GraphData> graphDatas = null;
        try {
            // 查询历史
            ReportHistoryResult cacheResult = reportHistoryResultRepository.query(history.getId(), paragraph.getId(),
                    contentTemplate.getId());
            if (cacheResult == null) {
                // 历史不存在，直接查相关数据库
                // switch (chartName) {
                // // 查询cpu使用率
                // case CPU_USEAGE_LINE_CHART_NAME:
                // graphDatas = monitorReportService.getUserCpuUsageLineChart(params,
                // frequenceType, resourceIds);
                // break;
                // // 查询内存使用率
                // case RAM_USE_LINE_CHART_NAME:
                // graphDatas = monitorReportService.getRamUseLineChart(params, frequenceType,
                // resourceIds);
                // break;
                // // 查询硬盘使用率
                // case DISK_USAGE_LINE_CHART_NAME:
                // graphDatas = monitorReportService.getDiskUsageLineChart(params,
                // frequenceType, resourceIds);
                // break;
                // // 查询硬盘IO
                // case DISK_IO_LINE_CHART_NAME:
                // graphDatas = monitorReportService.getDiskIOLineChart(params, frequenceType,
                // resourceIds);
                // break;
                // // 查询网络IO
                // case NET_IO_LINE_CHART_NAME:
                // graphDatas = monitorReportService.getNetIOLineChart(params, frequenceType,
                // resourceIds);
                // break;
                // // 查询系统load
                // case SYSTEM_LOAD_CHART_NAME:
                // graphDatas = monitorReportService.getSystemLoadLineChart(params,
                // frequenceType, resourceIds);
                // break;
                // default:
                // return null;
                // }
                graphDatas = Collections.emptyList();
                cacheResult = new ReportHistoryResult();
                String content = JSONObject.toJSONString(graphDatas);
                cacheResult.setContent(content);
                cacheResult.setHistoryId(history.getId());
                cacheResult.setContentTemplateId(contentTemplate.getId());
                cacheResult.setParagraphId(paragraph.getId());
                // 如果不存在则保存数据
                reportHistoryResultRepository.save(cacheResult);
            } else {
                graphDatas = JSONArray.parseArray(cacheResult.getContent(), GraphData.class);
            }
        } catch (Exception e) {
            logger.error(ExceptionUtils.getFullStackTrace(e));
        }
        switch (chartName) {
            // 查询cpu使用率
            case CPU_USEAGE_LINE_CHART_NAME:
                jasperPrint = new XyLineChartReportBuilder(graphDatas, "CPU使用率", "时间点", "使用率(%)").build();
                break;
            // 查询内存使用率
            case RAM_USE_LINE_CHART_NAME:
                jasperPrint = new XyLineChartReportBuilder(graphDatas, "内存使用率", "时间点", "使用率(%)").build();
                break;
            // 查询硬盘使用率
            case DISK_USAGE_LINE_CHART_NAME:
                jasperPrint = new XyLineChartReportBuilder(graphDatas, "硬盘使用率", "时间点", "使用率(%)").build();
                break;
            // 查询硬盘IO
            case DISK_IO_LINE_CHART_NAME:
                jasperPrint = new XyLineChartReportBuilder(graphDatas, "硬盘IO", "时间点", "速率").build();
                break;
            // 查询网络IO
            case NET_IO_LINE_CHART_NAME:
                jasperPrint = new XyLineChartReportBuilder(graphDatas, "网络IO", "时间点", "速率").build();
                break;
            // 查询系统load
            case SYSTEM_LOAD_CHART_NAME:
                jasperPrint = new XyLineChartReportBuilder(graphDatas, "系统负载", "时间点", "系统负载(%)").build();
                break;
            default:
                return null;

        }
        return jasperPrint;
    }
}
