package com.bocloud.cos.service.jasper.utils;

import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.engine.export.HtmlExporter;
import net.sf.jasperreports.engine.export.JRPdfExporter;
import net.sf.jasperreports.engine.export.ooxml.JRDocxExporter;
import net.sf.jasperreports.engine.export.ooxml.JRXlsxExporter;
import net.sf.jasperreports.export.*;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.OutputStream;
import java.util.List;

/**
 * 封装导出相关操作
 * 
 * @author hufy
 *
 */
public class JasperReportUtils {
	private static Logger logger = LoggerFactory.getLogger(JasperReportUtils.class);

	/**
	 * 输出报表html 字节流
	 * 
	 * @param reportPath
	 * @param list
	 * @param heads
	 * @return
	 */
	public static StringBuffer getHtmlBuffer(String reportPath, List<Object> list, List<String> heads) {
		JasperPrint jasperPrint = ReportToolUtils.dynFileReport(reportPath, list, heads);
		StringBuffer sbf = new StringBuffer();
		HtmlExporter exporter = new HtmlExporter();
		SimpleHtmlExporterConfiguration config = new SimpleHtmlExporterConfiguration();
		String header = "<html>\n" + "<head>\n" + "  <title></title>\n"
				+ "  <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"/>\n"
				+ "  <link rel=\"stylesheet\" type=\"text/css\" href=\"css/jasper.css\" />\n"
				+ "  <style type=\"text/css\">\n" + "    a {text-decoration: none}\n" + "  </style>\n" + "</head>\n"
				+ "<body text=\"#000000\" link=\"#000000\" alink=\"#000000\" vlink=\"#000000\" bgcolor=\"gray\">\n"
				+ "<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\">\n"
				+ "<tr><td width=\"50%\">&nbsp;</td><td align=\"center\">\n";
		config.setHtmlHeader(header);
		/**
		 * 设置每页之间没有空行
		 */
		config.setBetweenPagesHtml("");
		exporter.setConfiguration(config);
		exporter.setExporterInput(new SimpleExporterInput(jasperPrint));
		exporter.setExporterOutput(new SimpleHtmlExporterOutput(sbf));
		try {
			exporter.exportReport();
		} catch (JRException e) {
			logger.info("输出报表html格式失败：" + e.getStackTrace());
		}
		return sbf;
	}

	@SuppressWarnings("unchecked")
	public static void exportReportListToRtf(@SuppressWarnings("rawtypes") List jasperPrints,
			OutputStream outputStream) {
		JRDocxExporter jrDocxExporter = new JRDocxExporter();
		// 设置输入项
		ExporterInput exporterInput = new SimpleExporterInput(jasperPrints);
		jrDocxExporter.setExporterInput(exporterInput);

		// 设置输出项
		OutputStreamExporterOutput exporterOutput = new SimpleOutputStreamExporterOutput(outputStream);
		jrDocxExporter.setExporterOutput(exporterOutput);
		DocxReportConfiguration config = new SimpleDocxReportConfiguration();
		jrDocxExporter.setConfiguration(config);
		try {
			jrDocxExporter.exportReport();
			logger.debug("Genertate the RTF report ok! :" + outputStream);
		} catch (JRException e) {
			logger.error(ExceptionUtils.getFullStackTrace(e));
		}

	}

	@SuppressWarnings("unchecked")
	public static void exportReportListToExcel(@SuppressWarnings("rawtypes") List jasperPrints,
			OutputStream outputStream) {
		JRXlsxExporter xlsExporter = new JRXlsxExporter();
		// 设置输入项
		ExporterInput exporterInput = new SimpleExporterInput(jasperPrints);
		xlsExporter.setExporterInput(exporterInput);

		// 设置输出项
		OutputStreamExporterOutput exporterOutput = new SimpleOutputStreamExporterOutput(outputStream);
		xlsExporter.setExporterOutput(exporterOutput);

		try {
			xlsExporter.exportReport();
			logger.debug("Genertate the excel report ok! :" + outputStream);
		} catch (JRException e) {
			logger.error(ExceptionUtils.getFullStackTrace(e));
		}

	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	public static void exportReportListToPdf(List jasperPrints, OutputStream outputStream) {
		JRPdfExporter jrPdfExporter = new JRPdfExporter();
		// 设置输入项
		ExporterInput exporterInput = new SimpleExporterInput(jasperPrints);
		jrPdfExporter.setExporterInput(exporterInput);

		// 设置输出项
		OutputStreamExporterOutput exporterOutput = new SimpleOutputStreamExporterOutput(outputStream);
		jrPdfExporter.setExporterOutput(exporterOutput);

		try {
			jrPdfExporter.exportReport();
			logger.debug("Genertate the pdf report ok! :" + outputStream);
		} catch (JRException e) {
			logger.error(ExceptionUtils.getFullStackTrace(e));
		}

	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	public static void exportReportListToHtml(List jasperPrints, OutputStream outputStream) {
		HtmlExporter htmlExporter = new HtmlExporter();
		// 设置输入项
		ExporterInput exporterInput = new SimpleExporterInput(jasperPrints);
		htmlExporter.setExporterInput(exporterInput);

		// 设置输出项
		HtmlExporterOutput exporterOutput = new SimpleHtmlExporterOutput(outputStream);
		htmlExporter.setExporterOutput(exporterOutput);

		try {
			htmlExporter.exportReport();
			logger.debug("Genertate the html report ok! :" + outputStream);
		} catch (JRException e) {
			logger.error(ExceptionUtils.getFullStackTrace(e));
		}

	}

}
