package com.bocloud.cos.service.utils;

import com.bocloud.service.client.ServiceClient;
import org.apache.curator.framework.recipes.cache.NodeCache;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.nio.charset.StandardCharsets;

/**
 * 云服务供应商注册器
 *
 * @author wangyu
 * @version 1.0
 * @since 2017年9月22日
 */
@Component("configRegister")
public class ConfigRegister implements InitializingBean {
    private static final Logger logger = LoggerFactory.getLogger(ConfigRegister.class);
    private static NodeCache nodeCache;
    private final ServiceClient serviceClient;

    @Autowired
    public ConfigRegister(ServiceClient serviceClient) {
        this.serviceClient = serviceClient;
    }

    /**
     * 服务启动时将数据库中的云服务供应商信息注册到ZooKeeper中
     *
     * @throws Exception 系统异常
     */
    public void start() throws Exception {
        serviceClient.putSystem(null);
        // 处理ZooKeeper上的系统配置发生变化时的处理逻辑：更新内部的缓存
        nodeCache.getListenable().addListener(() -> {
            logger.info("system config data changed,Refresh them in cache!");
            Configuration.getInstance().update(new String(nodeCache.getCurrentData().getData(), StandardCharsets.UTF_8));
            logger.info("Refresh system config Success!");
        });
        logger.info("get system config data ,Refresh them in cache!");
        Configuration.getInstance().update(new String(nodeCache.getCurrentData().getData(), StandardCharsets.UTF_8));
        logger.info("get system config Success!");
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        nodeCache = new NodeCache(this.serviceClient.getClient().getClient(), serviceClient.getService().getSystem(), false);
        nodeCache.start(true);
        this.start();
    }

}
