package com.bocloud.cos.service.utils;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.JSONTools;
import com.bocloud.common.utils.ListTool;
import com.bocloud.cos.common.utils.DateFormatter;
import com.bocloud.service.client.ServiceClient;
import com.google.common.collect.Lists;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mail.MailException;
import org.springframework.mail.javamail.JavaMailSenderImpl;
import org.springframework.mail.javamail.MimeMessageHelper;
import org.springframework.stereotype.Component;

import javax.mail.MessagingException;
import javax.mail.internet.MimeMessage;
import java.io.File;
import java.util.*;

/**
 * 邮件发送器
 *
 * @author dmw
 */
@Component("emailReportSender")
public class EmailSender {

    private static Logger logger = LoggerFactory.getLogger(EmailSender.class);

    private String emailServer = "";

    private String userName = "";

    private String password = "";
    // ssl端口号
    private static final int port = 465;

    private JavaMailSenderImpl sender;

    private final ServiceClient client;

    @Autowired
    public EmailSender(final ServiceClient client) {
        this.client = client;
    }

    public Result send(String subject, String[] receiver, String mailContent, List<File> files) {
        sender = new JavaMailSenderImpl();
        try {
            // 获取zk上最新的邮箱配置信息
            String data = this.client.getSystem();
            JSONObject object = JSONTools.isJSONObj(data);
            if (null != object) {
                emailServer = object.getString("EmailServer");
                userName = object.getString("EmailUserName");
                password = object.getString("EmailPassword");
            }
        } catch (Exception exception) {
            logger.error("GET ZOOKEEPER DATE ERR:{}", ExceptionUtils.getFullStackTrace(exception));
        }
        sender.setHost(emailServer);
        sender.setUsername(userName);
        sender.setPassword(password);

        // 当发件人服务器为腾讯企业邮箱时需开启认证
        if (emailServer.contains("qq")) {
            sender.setPort(port);
            Properties properties = new Properties();
            // 开启认证
            properties.setProperty("mail.smtp.auth", "true");
            // 启用调试
            properties.setProperty("mail.debug", "false");
            // 设置链接超时
            properties.setProperty("mail.smtp.timeout", "25000");
            // 设置端口
            properties.setProperty("mail.smtp.port", Integer.toString(port));
            // 设置ssl端口
            properties.setProperty("mail.smtp.socketFactory.port", Integer.toString(port));
            properties.setProperty("mail.smtp.socketFactory.fallback", "false");
            properties.setProperty("mail.smtp.socketFactory.class", "javax.net.ssl.SSLSocketFactory");
            properties.setProperty("mail.smtp.ssl.enable", "true");
            sender.setJavaMailProperties(properties);
        }
        List<String> receivers = Lists.newArrayList();
        Collections.addAll(receivers, receiver);
        String sendHostName = StringUtils.substringBefore(StringUtils.substringAfter(userName, "@"), ".");
        Iterator<String> iterator = receivers.iterator();
        while (iterator.hasNext()) {
            String email = iterator.next();
            String receiverHostName = StringUtils.substringBefore(StringUtils.substringAfter(email, "@"), ".");
            if (sendHostName.equals(receiverHostName)) {
                try {
                    checkEmailAndSend(subject, email, files, null, mailContent);
                } catch (MessagingException e) {
                    logger.error("Email send exception: {}", ExceptionUtils.getFullStackTrace(e));
                    continue;
                } finally {
                    iterator.remove();
                }
            }
        }
        try {
            if (receivers.size() > 0) {
                checkEmailAndSend(subject, null, files, receivers.toArray(new String[receivers.size()]), mailContent);
            }
        } catch (MailException e) {
            logger.error("Email send exception: {}", ExceptionUtils.getFullStackTrace(e));
        } catch (MessagingException e) {
            logger.error("Email sender MessagingException: {}", ExceptionUtils.getFullStackTrace(e));
        }
        return new Result(true, "Email send success!");
    }

    private void checkEmailAndSend(String subject, String receiver, List<File> files, String[] receivers,
                                   String mailContent) throws MessagingException {
        // 建立HTML邮件消息
        MimeMessage mailMessage = sender.createMimeMessage();
        // true表示开始附件模式
        MimeMessageHelper messageHelper = new MimeMessageHelper(mailMessage, true, "utf-8");
        // 设置收件人，寄件人
        if (null != receiver) {
            messageHelper.setTo(receiver);
        } else {
            messageHelper.setTo(receivers);
        }
        messageHelper.setFrom(userName);
        messageHelper.setSubject(subject);
        if (!ListTool.isEmpty(files)) {
            for (int i = 0; i < files.size(); i++) {
                messageHelper.addAttachment(files.get(i).getName(), files.get(i));
            }
        }
        // true 表示启动HTML格式的邮件
        messageHelper.setText("<html><head></head><body>" + mailContent + "</h1></body></html>", true);
        Date beginDate = new Date(System.currentTimeMillis());
        logger.info(
                "========Email send begin========" + DateFormatter.formatToString(beginDate, "yyyy年MM月dd日_HH:mm:ss"));
        // 发送邮件
        try {
            sender.send(mailMessage);
        } catch (MailException e) {
            logger.error("Email sender MessagingException: {}", ExceptionUtils.getFullStackTrace(e));
        }
        Date endDate = new Date(System.currentTimeMillis());
        logger.info("========Email send end========" + DateFormatter.formatToString(endDate, "yyyy年MM月dd日_HH:mm:ss"));
    }
}